#include "TAS2HAMThread.h"
#include "TASApp.h"


TAS2HAMThread::TAS2HAMThread(wxThreadKind WXUNUSED(kind), TAS2HAMTileSpace* pSpace, wxSemaphore* pSemCounterBlock, wxSemaphore* pSemMainBlock, wxSemaphore* pSemFreeCount)
{
  m_pSpace            = pSpace;
  m_pSemCounterBlock  = pSemCounterBlock;
  m_pSemMainBlock     = pSemMainBlock;
  m_pSemFreeCount     = pSemFreeCount;
  m_bShouldDie        = false;
  m_min               = 0;
  m_max               = 0;
}

TAS2HAMThread::~TAS2HAMThread()
{

}

void* TAS2HAMThread::Entry()
{
  while (!GetShouldDie()) {
    m_pSemCounterBlock->Wait();
    if (!GetShouldDie()) {
      m_min = m_pSpace->m_currThreadMin;
      m_max = m_pSpace->m_currThreadMax;
      m_pSemMainBlock->Post();
      Iterate();
      m_pSemFreeCount->Post();
    }
  }
  Exit();
  return 0;
}

extern std::vector<int> TwoHAMHackSTDeleteList;
//std::vector<TASCoords> TwoHAMHackSTLocList;  //MJP TODO: perhaps use this to save locations of added supertiles for 2HAM hack
wxCriticalSection* g_pCritSectDeleteList = NULL;


bool TAS2HAMThread::Iterate()
{
  bool bAddedNew = false;

  for (size_t i=m_min; i<=m_max; i++) {
    size_t lowerBound = ((i < m_pSpace->m_vectSTS[i]->m_pST->m_minToCheck) ?
                         m_pSpace->m_vectSTS[i]->m_pST->m_minToCheck : i);
    //TASSupertile* pSTAdded = NULL;

    for (size_t j=lowerBound; j<m_pSpace->m_numSupertilesBefore; j++) {

      //  In the infinite count case, we can skip checking for combos if these supertiles have
      //  attempted to combine before
/*      if (((m_pSpace->m_vectSTS[i]->m_pST->m_count != TAS_2HAM_INFINITE_COUNT) ||
          (m_pSpace->m_vectSTS[j]->m_pST->m_count != TAS_2HAM_INFINITE_COUNT))
          && !m_pSpace->m_vectSTS[i]->m_pST->ComboAttempted(m_pSpace->m_vectSTS[j]->m_pST)) {
        // skip
      } else {*/
  //counter++;
        std::vector<TASSupertile*> vectCombos;
  //sFoo = wxString::Format(wxT("Combining supertile %d with %d. "), i, j);
  //g_pTASApp->SetSimDebugText(sFoo, true);
        if (m_pSpace->m_vectSTS[i]->m_pST->GetCombinations(m_pSpace->m_vectSTS[j]->m_pST, &vectCombos)) {
  //MJP TODO: add newly created supertile spaces for infinite count case.  Replace
  //          one of the existing input supertiles and delete the other, otherwise.
          TASSupertile* pDup = NULL;
          for (size_t k=0; k<vectCombos.size(); k++) {
            if ((m_pSpace->m_vectSTS[i]->m_pST->m_count == TAS_2HAM_INFINITE_COUNT) &&
                (m_pSpace->m_vectSTS[j]->m_pST->m_count == TAS_2HAM_INFINITE_COUNT) &&
                (pDup = m_pSpace->IsDuplicate(vectCombos[k])))
            {
              //  For each parent supertile, record the new child
              m_pSpace->m_vectSTS[i]->m_pST->m_setChildren.insert(pDup);
              m_pSpace->m_vectSTS[j]->m_pST->m_setChildren.insert(pDup);

              delete vectCombos[k];   //  This st was a duplicate, so delete it
            } else {
              bool bKeep = true;
              if (m_pSpace->m_bOnlyLargestKeep /*&&*/ ) {
//TODO!!!
              } else if ((m_pSpace->m_minKeepSize != 0) || (m_pSpace->m_maxKeepSize != 0)) {
                if ((vectCombos[k]->m_maxDim < m_pSpace->m_minKeepSize) || (vectCombos[k]->m_maxDim > m_pSpace->m_maxKeepSize)) {
                  bKeep = false;
                }
              } else {
                //  For each parent supertile, record the new child
                m_pSpace->m_vectSTS[i]->m_pST->m_setChildren.insert(vectCombos[k]);
                m_pSpace->m_vectSTS[j]->m_pST->m_setChildren.insert(vectCombos[k]);
              }

              if (bKeep) {
                size_t dims = m_pSpace->m_maxNewSupertile;
                if ((vectCombos[k]->m_maxDim + (2 * m_pSpace->m_defaultPadding)) > m_pSpace->m_maxNewSupertile) {
                  dims = vectCombos[k]->m_maxDim + (2 * m_pSpace->m_defaultPadding);
                }

                TASSupertileSpace* pSTS = new TASSupertileSpace(vectCombos[k], dims);
                m_pSpace->AddSupertileSpace(pSTS, m_pSpace->m_epoch+1);
                bAddedNew = true;

                //MJP: get rid of this stupid hack!!!
                if (g_pContext->m_bUse2HAMHack) {
                  wxCriticalSectionLocker lock(*g_pCritSectDeleteList);
                  if ((TwoHAMHackSTDeleteList[j] != TAS_2HAMHACK_DONT_DELETE) && (m_pSpace->m_vectSTS[i]->m_pST->m_numTiles <= 2)) {
                    TwoHAMHackSTDeleteList[j] = TAS_2HAMHACK_DELETE;
                  } else {
                    TwoHAMHackSTDeleteList[j] = TAS_2HAMHACK_DONT_DELETE;
                  }
                  if ((TwoHAMHackSTDeleteList[i] != TAS_2HAMHACK_DONT_DELETE) && (m_pSpace->m_vectSTS[j]->m_pST->m_numTiles <= 2)) {
                    TwoHAMHackSTDeleteList[i] = TAS_2HAMHACK_DELETE;
                  } else {
                    TwoHAMHackSTDeleteList[i] = TAS_2HAMHACK_DONT_DELETE;
                  }
                }

              }
            }
          }
        }
//      }
  //sFoo = wxT("\n");
  //g_pTASApp->SetSimDebugText(sFoo, true);
    }
    m_pSpace->m_vectSTS[i]->m_pST->m_minToCheck = m_pSpace->m_numSupertilesBefore;
  }
  return bAddedNew;
}

bool TAS2HAMThread::GetShouldDie()
{
  return m_bShouldDie;
}

void TAS2HAMThread::SetShouldDie()
{
  m_bShouldDie = true;
}