/////////////////////////////////////////////////////////////////////////////
// Name:        TASClientServer.h
// Purpose:     Defines classes used for client and server classes used for
//              multiple instances of TAS to be networked together to perform
//              distributed computing
// Author:      Matthew J. Patitz
// Copyright:   Copyright 2011, 2012 Matthew J. Patitz
// Licence:
//    This file is part of ISU TAS.
//
//    ISU TAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    ISU TAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with ISU TAS.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////////////

#include "TASClientServer.h"
#include "TASTiles.h"
#include "TASApp.h"
#include <wx/msgdlg.h>


TASConnection::TASConnection(void) : wxConnection() 
{

}


bool TASConnection::OnStartAdvise(const wxString &topic, const wxString &item)
{
  return true;
}

bool TASConnection::OnAdvise(const wxString& topic, const wxString& item, const void* data, size_t size, wxIPCFormat format)
{
//  wxMessageBox(topic, data);
  wxString sMsg = wxString::Format(_T("Advised by server, topic: '%s'\n"), topic.c_str());
  g_pTASApp->SetSimOutputText(sMsg, true);

  return true;
}

bool TASConnection::OnDisconnect(void)
{
  wxString sMsg = wxString::Format(_T("Disconnected\n"));
  g_pTASApp->SetSimOutputText(sMsg, true);

  return wxConnectionBase::OnDisconnect();
}

wxConnectionBase* TASClient::OnMakeConnection(void)
{
  TASConnection* pConn = new TASConnection();
  m_vectConns.push_back(pConn);

  return pConn;
}

bool TASConnectToServer(wxString sHostName, wxString sPort, wxString sTopic)
{
  // Create a new connection
  TASConnection* connection = (TASConnection*)g_pTASApp->m_client.MakeConnection(sHostName, sPort, sTopic);

  if (!connection) {
    wxString sMsg = wxString::Format(_T("Failed to connect to server '%s' on port '%s'\n"), &sHostName, &sPort);
    g_pTASApp->SetSimOutputText(sMsg, true);
    return false;
  } else {
    wxString sMsg = wxString::Format(_T("Successfully connected to server '%s' on port '%s'\n"), &sHostName, &sPort);
    g_pTASApp->SetSimOutputText(sMsg, true);
  }

  connection->StartAdvise(sTopic);
  return true;
}

TASServer::TASServer() : wxServer()
{
  m_sPort = TAS_SERVER_DEFAULT_PORT;
}

bool TASServer::Start()
{
  if (!Create(m_sPort)) {
    wxString sMsg = wxString::Format(_T("Failed to start server on port '%s'\n"), &m_sPort);
    g_pTASApp->SetSimOutputText(sMsg, true);
    return false;
  } else {
    wxString sMsg = wxString::Format(_T("Successfully started server on port '%s'\n"), &m_sPort);
    g_pTASApp->SetSimOutputText(sMsg, true);
  }

  return true;
}

TASConnection* TASServer::OnAcceptConnection(const wxString& sTopic)
{
  TASConnection* pConn = new TASConnection();
  m_vectConns.push_back(pConn);
  return pConn;
}
