/////////////////////////////////////////////////////////////////////////////
// Name:        TASDockers.h
// Purpose:     Declares all dockable window classes
// Author:      Matthew J. Patitz
// Copyright:   Copyright 2008, 2009 Matthew J. Patitz
// Licence:
//    This file is part of ISU TAS.
//
//    ISU TAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    ISU TAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with ISU TAS.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////////////

#ifndef TASDockers_H
#define TASDockers_H

#include <wx/dynarray.h>
#include <wx/button.h>
#include <wx/checkbox.h>
#include <wx/defs.h>
#include <wx/frame.h>
#include <wx/sizer.h>
#include <wx/radiobox.h>
#include <wx/radiobut.h>
#include <wx/popupwin.h>
#include <wx/scrolwin.h>
#include <wx/choice.h>
#include <wx/stattext.h>
#include <wx/textctrl.h>
#include <wx/glcanvas.h>
#include <wx/minifram.h>
#include <wx/gbsizer.h>

//#include "TAS.h"
//#include "TASFrmBase.h"
//#include "TASPanel.h"
#include "TAS3DCanvas.h"
#include "TASTileCanvas.h"

WX_DEFINE_ARRAY(wxButton*, ButtonArray);
WX_DEFINE_ARRAY(wxCheckBox*, CheckBoxArray);

class TAS3DAxesWin;

class TASDockableWin
{
public:
  virtual bool OnHide() = 0;
};

WX_DEFINE_ARRAY(wxStaticText*, StaticTextArray);
WX_DEFINE_ARRAY(wxChoice*,     ChoiceArray);

class TASTileTypeWin : public wxScrolledWindow, public TASDockableWin
{
public:
  TASTileTypeWin(TASEditorFrame* pEdFrm);
  ~TASTileTypeWin();

  void OnImageButton(wxCommandEvent& event);
  void OnApply(wxCommandEvent& event);
  void OnCancel(wxCommandEvent& event);
  bool OnHide();
  void OnPaint(wxPaintEvent& event);

  void OnChange(wxCommandEvent& event);
  void OnCPChange(wxColourPickerEvent& event);
  void UpdateTileTypeDef();
  
  void SetTemps();
  void PopulateLists();
  void SetCurrentTileType(int tileNum, bool bForce=false);
  void DrawCurrentTileType(wxDC* pDC);
  void EnableEditing(bool bEnable = true);
  void SetConcentrationVisibility();
  int GetCurrentDimMode();

  wxBoxSizer*               m_pSizerMain;
  wxBoxSizer*               m_pSizerConcentration;
  wxTextCtrl*               m_pNameText;
  wxTextCtrl*               m_pConcentrationText;
  wxComboBox*               m_pLabelBox;
  std::vector<wxChoice*>    m_pStrengthBoxes;
  std::vector<wxComboBox*>  m_pGlueBoxes;
  wxRadioBox*               m_pRadTileColor;
  wxRadioBox*               m_pRadTextColor;
  wxRadioBox*               m_pRadImage;
  wxColourPickerCtrl*       m_pTileColorPicker;
  wxColourPickerCtrl*       m_pTextColorPicker;
  wxButton*                 m_pButtonImage;

  TASEditorFrame*           m_pParentFrame;
  TASTileType*              m_pTT;                //  Pointer to the tiletype being edited
  wxString                  m_sImagePath;
  int                       m_tileNum;
  wxSize                    m_size3DTiles;

private:
  wxArrayString             m_arrStrengths;
  bool                      m_bInSet;

  DECLARE_CLASS(TASTileTypeWin)
  DECLARE_EVENT_TABLE()
};

class TAS2HAMConfigWin : public wxScrolledWindow, public TASDockableWin
{
public:
  TAS2HAMConfigWin(TASSimulatorFrame* parent);
  ~TAS2HAMConfigWin();

  void OnViewApply(wxCommandEvent& event);
  void OnKeepApply(wxCommandEvent& event);
  void OnThreadApply(wxCommandEvent& event);
  bool OnHide();

  void PopulateSupertileStats();
  void SetProgress(int percent);

  TASSimulatorFrame*     m_parent;

  wxStaticBoxSizer* m_pStatBoxView;
  wxStaticText*     m_pStatTextViewMin;
  wxTextCtrl*       m_pTextCtrlViewMin;
  wxStaticText*     m_pStatTextViewMax;
  wxTextCtrl*       m_pTextCtrlViewMax;
  wxCheckBox*       m_pCheckView;
  wxButton*         m_pButtView;

  wxStaticBoxSizer* m_pStatBoxKeep;
  wxStaticText*     m_pStatTextKeepMin;
  wxTextCtrl*       m_pTextCtrlKeepMin;
  wxStaticText*     m_pStatTextKeepMax;
  wxTextCtrl*       m_pTextCtrlKeepMax;
  wxCheckBox*       m_pCheckKeep;
  wxButton*         m_pButtKeep;
  wxGauge*          m_pProgressGauge;

  wxStaticBoxSizer* m_pStatBoxThreads;
  wxStaticText*     m_pStatTextThreads;
  wxTextCtrl*       m_pTextCtrlThreads;
  wxButton*         m_pButtThreads;

  wxStaticBoxSizer* m_pStaticBoxSupertiles;

  DECLARE_CLASS(TAS2HAMConfigWin)
  DECLARE_EVENT_TABLE()
};

class TAS3DCellValueWin : public wxScrolledWindow, public TASDockableWin
{
public:
  TAS3DCellValueWin(TASSimulatorFrame* parent);
  ~TAS3DCellValueWin();
  bool OnHide();
  void OnSize(wxSizeEvent& event);
  void OnPaint(wxPaintEvent& event);
  void OnClose(wxCloseEvent& event);
  void OnCoordChange(wxCommandEvent& event);
  void OnValueChange(wxCommandEvent& event);
  void OnApply(wxCommandEvent& event);
  void OnCenter(wxCommandEvent& event);
  
  void SetCurrentCell(TASCoords cellCoords, bool bFieldsOnly, bool bFixed);
  TASCoords& GetCurrentCell();
  void SetFields();
  void SetConcentrationVisibility();
  void SetSupertileInfoVisibility();

  void DrawCurrentTileType(wxDC* pDC);
  
private:
  void SetupVertLayout();
  void SetupHorLayout();

  TASCoords         m_coordsCurrCell;
  TASSimulatorFrame*         m_parent;

  wxArrayString     m_arrNames;

  wxComboBox*       m_pComboBoxNames;
  StaticTextArray   m_arrStatTexts;
  wxStaticText*     m_pCoordText;
  wxStaticText*     m_pTileConcText;
  wxStaticText*     m_pTileOrderText;
  wxStaticText*     m_pSupertileText;
  wxStaticText*     m_pChildrenText;
  wxTextCtrl*       m_pTextCtrlX;
  wxTextCtrl*       m_pTextCtrlY;
  wxTextCtrl*       m_pTextCtrlZ;
  wxButton*         m_pButtonApply;
  wxButton*         m_pButtonCenter;

  wxBoxSizer*       m_pVertSizer;
  wxBoxSizer*       m_pHorSizer;

  wxSize            m_size3DTiles;

  DECLARE_CLASS(TAS3DCellValueWin)
  DECLARE_EVENT_TABLE()
};

class TAS3DCASpaceWin : public wxScrolledWindow, public TASDockableWin
{
public:
  TAS3DCASpaceWin(TASSimulatorFrame* parent);
  ~TAS3DCASpaceWin();

  bool OnHide();
  void OnSize(wxSizeEvent& event);
  void OnClose(wxCloseEvent& event);
  void OnViewTypeChange(wxCommandEvent& event);
  void OnApply(wxCommandEvent& event);
  void OnReset(wxCommandEvent& event);
  
  void SetCurrentDims();
  void SetViewType(int viewType);

  TASSimulatorFrame*   m_parent;

  //  Controls for displaying current space dimensions
  wxStaticBoxSizer* m_pSizerBoxDims;
  wxGridBagSizer*   m_pGBSizerDims;
  wxStaticText*     m_pTextDimX;
  wxStaticText*     m_pTextDimY;
  wxStaticText*     m_pTextDimZ;
  wxStaticText*     m_pTextDimMinX;
  wxStaticText*     m_pTextDimMinY;
  wxStaticText*     m_pTextDimMinZ;
  wxStaticText*     m_pTextDimMaxX;
  wxStaticText*     m_pTextDimMaxY;
  wxStaticText*     m_pTextDimMaxZ;

  //  Controls for determining what portion of the space is displayed
  wxRadioButton*    m_pRadioViewBy1;
  wxRadioButton*    m_pRadioViewBy2;
  wxRadioButton*    m_pRadioViewBy3;

  //  Controls for setting a sub-range of the space to display
  wxGridBagSizer*   m_pGBSizerRange;
  wxStaticText*     m_pTextRangeX;
  wxStaticText*     m_pTextRangeY;
  wxStaticText*     m_pTextRangeZ;
  wxStaticText*     m_pTextRangeDashX;
  wxStaticText*     m_pTextRangeDashY;
  wxStaticText*     m_pTextRangeDashZ;
  wxTextCtrl*       m_pTextCtrlRangeMinX;
  wxTextCtrl*       m_pTextCtrlRangeMinY;
  wxTextCtrl*       m_pTextCtrlRangeMinZ;
  wxTextCtrl*       m_pTextCtrlRangeMaxX;
  wxTextCtrl*       m_pTextCtrlRangeMaxY;
  wxTextCtrl*       m_pTextCtrlRangeMaxZ;

  //  Controls for setting a neighborhood in the space to display
  wxGridBagSizer*   m_pGBSizerNbrhd;
  wxStaticText*     m_pTextNbrhdX;
  wxStaticText*     m_pTextNbrhdY;
  wxStaticText*     m_pTextNbrhdZ;
  wxTextCtrl*       m_pTextCtrlNbrhdX;
  wxTextCtrl*       m_pTextCtrlNbrhdY;
  wxTextCtrl*       m_pTextCtrlNbrhdZ;

  wxBoxSizer*       m_pSizerButtons;
  wxButton*         m_pButtonApply;
  wxButton*         m_pButtonReset;

  wxStaticBoxSizer* m_pHSizerBoxViewBy;
  wxStaticBoxSizer* m_pVSizerBoxViewBy;

private:
  void SetupVertLayout();
  void SetupHorLayout();
  int  GetCoordValueFromString(wxString sVal, int type);
  wxString GetCoordStringFromValue(int iVal);

  wxBoxSizer*   m_pMainSizer;
  wxBoxSizer*   m_pVertSizer;
  wxBoxSizer*   m_pHorSizer;
  int           m_orient;

  DECLARE_CLASS(TAS3DCASpaceWin)
  DECLARE_EVENT_TABLE()
};

class TAS3DAxesCanvas : public wxGLCanvas
{
public:
  TAS3DAxesCanvas(TAS3DAxesWin* parent);
  ~TAS3DAxesCanvas();

  void OnPaint(wxPaintEvent& event);
  void OnSize(wxSizeEvent& event);
  void OnEraseBackground(wxEraseEvent& event);

  void RotateAxes(GLfloat* pQuat);

  bool IsValid();
  void InitGL();

  TAS3DAxesWin*   m_parent;
  bool              m_bDoInit;            //  True if initgl not yet called
  GLfloat*          m_pfQuat;

private:
  DECLARE_EVENT_TABLE()
};

class TAS3DAxesWin : public wxScrolledWindow, public TASDockableWin
{
public:
  TAS3DAxesWin(TASSimulatorFrame* parent, TAS3DPanel* panel, wxPoint ptLocation = wxDefaultPosition, wxSize szSize = wxSize(200, 200));
  ~TAS3DAxesWin();

  bool OnHide();
  void OnSize(wxSizeEvent& event);
  void RotateAxes(GLfloat* pQuat) { m_pCanvas->RotateAxes(pQuat); };

  TASSimulatorFrame*           m_parent;
  TAS3DPanel*         m_panel;
  TAS3DAxesCanvas*    m_pCanvas;

private:
  DECLARE_CLASS(TAS3DAxesWin)
  DECLARE_EVENT_TABLE()
};

class TAS3DOVCanvas : public TAS3DCanvas
{
public:
  TAS3DOVCanvas(wxScrolledWindow* parent, TAS3DPanel* pMainPanel, TASSimulatorFrame* parentFrame);

  void OnMouse(wxMouseEvent& event);
  void DrawOverlay();

private:
  DECLARE_EVENT_TABLE()
};

class TASOverviewRect
{
public:
  TASOverviewRect(wxWindow* pWin, int width);

  void Draw(wxDC* pDC, wxRect& rect, bool bOnlyDraw);
  void Clear(wxDC* pDC);

protected:
  wxWindow*   m_pWin;
  wxRect      m_oldRect;
  wxPen       m_pen;
};

class TAS3DOverviewWin : public wxScrolledWindow, public TASDockableWin
{
public:
  TAS3DOverviewWin(TASSimulatorFrame* parent, TAS3DPanel* panel, wxPoint ptLocation = wxDefaultPosition, wxSize szSize = wxSize(200, 200));
  ~TAS3DOverviewWin();

  bool OnHide();
  void OnSize(wxSizeEvent& event);

  TASSimulatorFrame*          m_parent;
  TAS3DOVCanvas*              m_pCanvas;
  TASOverviewRect             m_rectOverlay;

private:
  DECLARE_CLASS(TAS3DOverviewWin)
  DECLARE_EVENT_TABLE()
};

class TASTileOverviewWin : public TASTilePanel, public TASDockableWin
{
public:
  TASTileOverviewWin(TASSimulatorFrame* parent, TASTilePanel* panel, wxPoint ptLocation = wxDefaultPosition, wxSize szSize = wxSize(200, 200));
  ~TASTileOverviewWin();

  void Refresh( bool eraseBackground = true,
                const wxRect *rect = (const wxRect *) NULL );

  bool OnHide();
  void OnSize(wxSizeEvent& event);
  void OnPaint(wxPaintEvent& event);

  void OnMouse(wxMouseEvent& event);
  void DrawOverlay(wxDC* pDC, bool bOnlyDraw);
  void ClearOverlay(wxDC* pDC) { m_rectOverlay.Clear(pDC); };
  void DoScaling(wxDC* pDC);
  float GetScaleFact() { return m_pMainPanel->GetScaleFact(); };
  int GetVirtHeight() { return m_pMainPanel->GetVirtHeight(); };

  void GetCoordsFromLocation(int x, int y, TASCoords* coords, bool bUseTilespaceDims = false);
  void GetTileLocation(TASCoords& coords, int* x, int* y);

  inline int GetTileWidth(int col);
  inline int GetTileHeight(int row);

  TASSimulatorFrame*     m_parent;
  TASOverviewRect        m_rectOverlay;

private:
  int                    m_widthMod;
  int                    m_heightMod;

  DECLARE_CLASS(TASTileOverviewWin)
  DECLARE_EVENT_TABLE()
};

class TASNotebookWin : public wxAuiNotebook, public TASDockableWin
{
public:
  TASNotebookWin(TASFrameBase* parent, wxWindowID id, wxMenu** pPMenu, int iMenuItemID, const wxPoint& pos, const wxSize& size, long style);
  ~TASNotebookWin();

  bool OnHide();

private:
  TASFrameBase* m_parent;
  wxMenu**      m_pPMenu;
  int           m_iMenuItemID;

  DECLARE_CLASS(TASNotebookWin)
  DECLARE_EVENT_TABLE()
};

#endif // TASDockers_H


