/////////////////////////////////////////////////////////////////////////////
// Name:        TASEditor.cpp
// Purpose:     Defines frame and other classes used by editor
// Author:      Matthew J. Patitz
// Copyright:   Copyright 2008, 2009 Matthew J. Patitz
// Licence:
//    This file is part of ISU TAS.
//
//    ISU TAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    ISU TAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with ISU TAS.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#include "wxprec.h"
#include <wx/aui/aui.h>

#include "TASEditor.h"
#include "TASApp.h"
#include "TASTiles.h"
#include "TAS3DCanvas.h"
#include "TASDockers.h"
#include "TASTilePanel.h"

#include <stdlib.h>

#include <wx/sizer.h>
#include <wx/textctrl.h>
#include <wx/menu.h>
#include <wx/filename.h>
#include <wx/filedlg.h>
#include <wx/msgdlg.h>
#include <wx/textdlg.h>
#include <wx/statline.h>
#include <wx/arrimpl.cpp>
#include <wx/xrc/xmlres.h>
#include <wx/utils.h>
//#include <wx/svg/dcsvg.h>

#include "TASDebug.h"


TASTiletypePanel::TASTiletypePanel() {
  m_pEditor = NULL;
}

TASTiletypePanel::~TASTiletypePanel() {
  delete m_pEditor;
  m_pEditor = NULL;
}

/*
IMPLEMENT_CLASS(TASEditorTextCtrl, wxStyledTextCtrl)
BEGIN_EVENT_TABLE(TASEditorTextCtrl, wxStyledTextCtrl)
  EVT_CHAR(                         TASEditorTextCtrl::OnChar)
//  EVT_STC_MODIFIED(wxID_ANY,        TASEditorTextCtrl::OnTextMod)
  EVT_STC_CHANGE(wxID_ANY,          TASEditorTextCtrl::OnTextMod)
  EVT_MENU(wxID_CLEAR,              TASEditorTextCtrl::OnEditClear)
  EVT_MENU(wxID_CUT,                TASEditorTextCtrl::OnEditCut)
  EVT_MENU(wxID_COPY,               TASEditorTextCtrl::OnEditCopy)
  EVT_MENU(wxID_PASTE,              TASEditorTextCtrl::OnEditPaste)
  EVT_MENU(Menu_Edit_ID_INDENTINC,  TASEditorTextCtrl::OnEditIndentInc)
  EVT_MENU(Menu_Edit_ID_INDENTRED,  TASEditorTextCtrl::OnEditIndentRed)
  EVT_MENU(wxID_SELECTALL,          TASEditorTextCtrl::OnEditSelectAll)
  EVT_MENU(Menu_Edit_ID_SELECTLINE, TASEditorTextCtrl::OnEditSelectLine)
  EVT_MENU(wxID_REDO,               TASEditorTextCtrl::OnEditRedo)
  EVT_MENU(wxID_UNDO,               TASEditorTextCtrl::OnEditUndo)
  EVT_MENU(Menu_Edit_ID_BRACEMATCH, TASEditorTextCtrl::OnBraceMatch)
  EVT_MENU(Menu_Edit_ID_COMMENT,    TASEditorTextCtrl::OnCommentSelection)
  EVT_MENU(Menu_Edit_ID_UNCOMMENT,  TASEditorTextCtrl::OnUncommentSelection)
  EVT_STC_MARGINCLICK(wxID_ANY,     TASEditorTextCtrl::OnMarginClick)
  EVT_KEY_DOWN(TASEditorTextCtrl::OnKey)
END_EVENT_TABLE()
*/
  
IMPLEMENT_CLASS(TASEditorFrame, wxFrame)
BEGIN_EVENT_TABLE(TASEditorFrame, wxFrame)
  EVT_MENU(Menu_File_NewTileset,              TASEditorFrame::OnNewTileset)
  EVT_MENU(Menu_File_LoadTileset,             TASEditorFrame::OnLoadTileset)
  EVT_MENU(Menu_File_ReloadTileset,           TASEditorFrame::OnReloadTileset)
  EVT_MENU(Menu_File_SaveTileset,             TASEditorFrame::OnSaveTileset)
  EVT_MENU(Menu_File_SaveTilesetAs,           TASEditorFrame::OnSaveTilesetAs)
  EVT_MENU(Menu_File_TilesetImages,           TASEditorFrame::OnSaveTilesetImages)
  EVT_MENU(Menu_File_Quit,                    TASSimulatorFrame::OnQuit)

  EVT_MENU(Menu_View_ToggleShowMessages,      TASEditorFrame::OnToggleShowMessages)
  EVT_MENU(Menu_View_ToggleShowTTEditor,      TASEditorFrame::OnToggleShowTTEditor)
  EVT_MENU(Menu_View_ToggleShowSymbols,       TASEditorFrame::OnToggleShowSymbols)
  EVT_MENU(Menu_View_ToggleShowTileNames,     TASEditorFrame::OnToggleShowTileNames)
  EVT_MENU(Menu_View_ToggleShowTileGlues,     TASEditorFrame::OnToggleShowTileGlues)

  EVT_MENU(Menu_Help,                         TASFrameBase::OnHelp)

  EVT_MENU(wxID_PRINT_SETUP,                  TASEditorFrame::OnPrintSetup)
  EVT_MENU(wxID_PREVIEW,                      TASEditorFrame::OnPrintPreview)
  EVT_MENU(wxID_PRINT,                        TASEditorFrame::OnPrint)
  EVT_MENU(Menu_Tileset_IntoSimulator,        TASEditorFrame::OnTilesetToSimulator)
  EVT_MENU(wxID_CLEAR,                        TASEditorFrame::OnEdit)
  EVT_MENU(wxID_CUT,                          TASEditorFrame::OnEdit)
  EVT_MENU(wxID_COPY,                         TASEditorFrame::OnEdit)
  EVT_MENU(wxID_PASTE,                        TASEditorFrame::OnEdit)
  EVT_MENU(Menu_Edit_ID_INDENTINC,            TASEditorFrame::OnEdit)
  EVT_MENU(Menu_Edit_ID_INDENTRED,            TASEditorFrame::OnEdit)
  EVT_MENU(wxID_SELECTALL,                    TASEditorFrame::OnEdit)
  EVT_MENU(Menu_Edit_ID_SELECTLINE,           TASEditorFrame::OnEdit)
  EVT_MENU(wxID_REDO,                         TASEditorFrame::OnEdit)
  EVT_MENU(wxID_UNDO,                         TASEditorFrame::OnEdit)
  EVT_MENU(Menu_Edit_ID_BRACEMATCH,           TASEditorFrame::OnEdit)
  EVT_MENU(Menu_Edit_ID_COMMENT,              TASEditorFrame::OnEdit)
  EVT_MENU(Menu_Edit_ID_UNCOMMENT,            TASEditorFrame::OnEdit)
  // find
/*
  EVT_MENU(wxID_FIND,                         TASEditorFrame::OnFind)
  EVT_MENU(Menu_Edit_ID_FINDNEXT,             TASEditorFrame::OnFindNext)
  EVT_MENU(Menu_Edit_ID_FINDPREV,             TASEditorFrame::OnFindPrev)
  EVT_MENU(Menu_Edit_ID_REPLACE,              TASEditorFrame::OnReplace)
  EVT_MENU(Menu_Edit_ID_REPLACENEXT,          TASEditorFrame::OnReplaceNext)
  
  EVT_MENU(Menu_Edit_ID_BOOKMARKADD,          TASEditorFrame::OnBookmarkAdd)
  EVT_MENU(Menu_Edit_ID_BOOKMARKREM,          TASEditorFrame::OnBookmarkRem)
  EVT_MENU(Menu_Edit_ID_NEXTBOOKMARK,         TASEditorFrame::OnNextBookmark)
  EVT_MENU(Menu_Edit_ID_PREVBOOKMARK,         TASEditorFrame::OnPrevBookmark)

  EVT_FIND(wxID_ANY,                          TASEditorFrame::OnFindDialog)
  EVT_FIND_NEXT(wxID_ANY,                     TASEditorFrame::OnFindDialog)
  EVT_FIND_REPLACE(wxID_ANY,                  TASEditorFrame::OnFindDialog)
  EVT_FIND_REPLACE_ALL(wxID_ANY,              TASEditorFrame::OnFindDialog)
  EVT_FIND_CLOSE(wxID_ANY,                    TASEditorFrame::OnFindDialog)
*/

  //  Tile editor menu items
  EVT_MENU_RANGE(ID_TAS_TILEED_EDIT, ID_TAS_TILEED_EDIT + 60, TASEditorFrame::OnTileEditorEditMenu)

  EVT_COMMAND(ID_TAS_EDITOR_FRAME, tasEVT_POST_SETOUTPUTTEXT,  TASFrameBase::OnPostSetOutputText)
  EVT_COMMAND(ID_TAS_EDITOR_FRAME, tasEVT_POST_SETDEBUGTEXT,   TASFrameBase::OnPostSetDebugText)

  EVT_CLOSE(TASEditorFrame::OnClose)
END_EVENT_TABLE()

int g_iMarkerCounter = 0;

/*
TASEditorTextCtrl::TASEditorTextCtrl(wxWindow* parent, TASEditorFrame *parentFrame,
                                           wxWindowID id, const wxString& WXUNUSED(value),
                                           const wxPoint& pos, const wxSize& size, long style,
                                           const wxValidator& WXUNUSED(validator), const wxString& name)
                     : wxStyledTextCtrl(parent, id, pos, size, style, name)
{
  m_parentFrame = parentFrame;
  m_LineNrID  = 0;
  m_DividerID = 1;
  m_FoldingID = 2;

  // initialize language
  m_language = NULL;

  // default font for all styles
  SetViewEOL(g_CommonPrefs.displayEOLEnable);
  SetIndentationGuides (g_CommonPrefs.indentGuideEnable);
  SetEdgeMode(g_CommonPrefs.longLineOnEnable?
              wxSTC_EDGE_LINE: wxSTC_EDGE_NONE);
  SetViewWhiteSpace (g_CommonPrefs.whiteSpaceEnable?
                     wxSTC_WS_VISIBLEALWAYS: wxSTC_WS_INVISIBLE);
  SetOvertype(g_CommonPrefs.overTypeInitial);
  SetReadOnly(g_CommonPrefs.readOnlyInitial);
  SetWrapMode(g_CommonPrefs.wrapModeInitial?
              wxSTC_WRAP_WORD: wxSTC_WRAP_NONE);
  wxFont font(10, wxMODERN, wxNORMAL, wxNORMAL);
  StyleSetFont(wxSTC_STYLE_DEFAULT, font);
  StyleSetForeground(wxSTC_STYLE_DEFAULT, *wxBLACK);
  StyleSetBackground(wxSTC_STYLE_DEFAULT, *wxWHITE);
  StyleSetForeground(wxSTC_STYLE_LINENUMBER, wxColour (_T("DARK GREY")));
  StyleSetBackground(wxSTC_STYLE_LINENUMBER, *wxWHITE);
  StyleSetForeground(wxSTC_STYLE_INDENTGUIDE, wxColour (_T("DARK GREY")));
//  InitializePrefs("C++");

  // set visibility
  SetVisiblePolicy(wxSTC_VISIBLE_STRICT|wxSTC_VISIBLE_SLOP, 1);
  SetXCaretPolicy(wxSTC_CARET_EVEN|wxSTC_VISIBLE_STRICT|wxSTC_CARET_SLOP, 1);
  SetYCaretPolicy(wxSTC_CARET_EVEN|wxSTC_VISIBLE_STRICT|wxSTC_CARET_SLOP, 1);

  // markers
  MarkerDefine(wxSTC_MARKNUM_FOLDER,        wxSTC_MARK_DOTDOTDOT, _T("BLACK"), _T("BLACK"));
  MarkerDefine(wxSTC_MARKNUM_FOLDEROPEN,    wxSTC_MARK_ARROWDOWN, _T("BLACK"), _T("BLACK"));
  MarkerDefine(wxSTC_MARKNUM_FOLDERSUB,     wxSTC_MARK_EMPTY,     _T("BLACK"), _T("BLACK"));
  MarkerDefine(wxSTC_MARKNUM_FOLDEREND,     wxSTC_MARK_DOTDOTDOT, _T("BLACK"), _T("WHITE"));
  MarkerDefine(wxSTC_MARKNUM_FOLDEROPENMID, wxSTC_MARK_ARROWDOWN, _T("BLACK"), _T("WHITE"));
  MarkerDefine(wxSTC_MARKNUM_FOLDERMIDTAIL, wxSTC_MARK_EMPTY,     _T("BLACK"), _T("BLACK"));
  MarkerDefine(wxSTC_MARKNUM_FOLDERTAIL,    wxSTC_MARK_EMPTY,     _T("BLACK"), _T("BLACK"));

  // create a marker type for bookmarks
  MarkerDefine(1, wxSTC_MARK_ROUNDRECT, _T("CYAN"), _T("CYAN"));//wxNullColour, *wxCYAN);

  // miscellaneous
  m_LineNrMargin = TextWidth(wxSTC_STYLE_LINENUMBER, _T("_999999"));
  m_FoldingMargin = 16;
  CmdKeyClear(wxSTC_KEY_TAB, 0); // this is done by the menu accelerator key
  SetLayoutCache(wxSTC_CACHE_PAGE);

  SetUndoCollection(true);
}

void TASEditorTextCtrl::OnTextMod(wxStyledTextEvent& WXUNUSED(event))
{
//  EndUndoAction();
//  BeginUndoAction();
  m_parentFrame->SetTabTitle();
}

void TASEditorTextCtrl::OnChar(wxKeyEvent& event)
{
  wxCommandEvent evt;
  if (event.GetKeyCode() == WXK_F2 && event.ControlDown()) {
    if (MarkerGet(GetCurrentLine())) {
      m_parentFrame->OnBookmarkRem(evt);
    } else {
      m_parentFrame->OnBookmarkAdd(evt);
    }
  } else if (event.GetKeyCode() == WXK_F2 && event.ShiftDown()) {
    m_parentFrame->OnPrevBookmark(evt);
  } else if (event.GetKeyCode() == WXK_F2) {
    m_parentFrame->OnNextBookmark(evt);
  } else if (event.GetKeyCode() == WXK_F3 && event.ControlDown()) {
    wxString selText = GetSelectedText();
    if (selText.Length() > 0) {
      m_parentFrame->m_findData->SetFindString(selText);

      if (event.ShiftDown()) {
        m_parentFrame->m_findData->SetFlags(0);
      } else {
        m_parentFrame->m_findData->SetFlags(wxFR_DOWN);
      }
      m_parentFrame->FindNext();
    }
  } else {
    event.Skip();
  }
}

void TASEditorTextCtrl::SetSavePointEx()
{
  wxStyledTextCtrl::SetSavePoint();
  m_parentFrame->SetTabTitle();
}

bool TASEditorTextCtrl::InitializePrefs(const wxString &name) {

    // initialize styles
    StyleClearAll();
    LanguageInfo const* curInfo = NULL;

    // determine language
    bool found = false;
    int languageNr;
    for (languageNr = 0; languageNr < g_LanguagePrefsSize; languageNr++) {
        curInfo = &g_LanguagePrefs [languageNr];
        if (curInfo->name == name) {
            found = true;
            break;
        }
    }
    if (!found) return false;

    // set lexer and language
    SetLexer (curInfo->lexer);
    m_language = curInfo;

    // set margin for line numbers
    SetMarginType (m_LineNrID, wxSTC_MARGIN_NUMBER);
    StyleSetForeground (wxSTC_STYLE_LINENUMBER, wxColour (_T("DARK GREY")));
    StyleSetBackground (wxSTC_STYLE_LINENUMBER, *wxWHITE);
    SetMarginWidth (m_LineNrID, 0); // start out not visible

    // default fonts for all styles!
    int Nr;
    for (Nr = 0; Nr < wxSTC_STYLE_LASTPREDEFINED; Nr++) {
        wxFont font (10, wxMODERN, wxNORMAL, wxNORMAL);
        StyleSetFont (Nr, font);
    }

    // set common styles
    StyleSetForeground (wxSTC_STYLE_DEFAULT, wxColour (_T("DARK GREY")));
    StyleSetForeground (wxSTC_STYLE_INDENTGUIDE, wxColour (_T("DARK GREY")));

    // initialize settings
    if (g_CommonPrefs.syntaxEnable) {
        int keywordnr = 0;
        for (Nr = 0; Nr < STYLE_TYPES_COUNT; Nr++) {
            if (curInfo->styles[Nr].type == -1) continue;
            const StyleInfo &curType = g_StylePrefs [curInfo->styles[Nr].type];
            wxFont font (curType.fontsize, wxMODERN, wxNORMAL, wxNORMAL, false,
                         curType.fontname);
            StyleSetFont (Nr, font);
            if (curType.foreground) {
                StyleSetForeground (Nr, wxColour (curType.foreground));
            }
            if (curType.background) {
                StyleSetBackground (Nr, wxColour (curType.background));
            }
            StyleSetBold (Nr, (curType.fontstyle & mySTC_STYLE_BOLD) > 0);
            StyleSetItalic (Nr, (curType.fontstyle & mySTC_STYLE_ITALIC) > 0);
            StyleSetUnderline (Nr, (curType.fontstyle & mySTC_STYLE_UNDERL) > 0);
            StyleSetVisible (Nr, (curType.fontstyle & mySTC_STYLE_HIDDEN) == 0);
            StyleSetCase (Nr, curType.lettercase);
            const wxChar *pwords = curInfo->styles[Nr].words;
            if (pwords) {
                SetKeyWords (keywordnr, pwords);
                keywordnr += 1;
            }
        }
    }

    // set margin as unused
    SetMarginType (m_DividerID, wxSTC_MARGIN_SYMBOL);
    SetMarginWidth (m_DividerID, 0);
    SetMarginSensitive (m_DividerID, false);

    // folding
    SetMarginType (m_FoldingID, wxSTC_MARGIN_SYMBOL);
    SetMarginMask (m_FoldingID, wxSTC_MASK_FOLDERS);
    StyleSetBackground (m_FoldingID, *wxWHITE);
    SetMarginWidth (m_FoldingID, 0);
    SetMarginSensitive (m_FoldingID, false);
    if (g_CommonPrefs.foldEnable) {
        SetMarginWidth (m_FoldingID, curInfo->folds != 0? m_FoldingMargin: 0);
        SetMarginSensitive (m_FoldingID, curInfo->folds != 0);
        SetProperty (_T("fold"), curInfo->folds != 0? _T("1"): _T("0"));
        SetProperty (_T("fold.comment"),
                     (curInfo->folds & mySTC_FOLD_COMMENT) > 0? _T("1"): _T("0"));
        SetProperty (_T("fold.compact"),
                     (curInfo->folds & mySTC_FOLD_COMPACT) > 0? _T("1"): _T("0"));
        SetProperty (_T("fold.preprocessor"),
                     (curInfo->folds & mySTC_FOLD_PREPROC) > 0? _T("1"): _T("0"));
        SetProperty (_T("fold.html"),
                     (curInfo->folds & mySTC_FOLD_HTML) > 0? _T("1"): _T("0"));
        SetProperty (_T("fold.html.preprocessor"),
                     (curInfo->folds & mySTC_FOLD_HTMLPREP) > 0? _T("1"): _T("0"));
        SetProperty (_T("fold.comment.python"),
                     (curInfo->folds & mySTC_FOLD_COMMENTPY) > 0? _T("1"): _T("0"));
        SetProperty (_T("fold.quotes.python"),
                     (curInfo->folds & mySTC_FOLD_QUOTESPY) > 0? _T("1"): _T("0"));
    }
    SetFoldFlags (wxSTC_FOLDFLAG_LINEBEFORE_CONTRACTED |
                  wxSTC_FOLDFLAG_LINEAFTER_CONTRACTED);

    // set spaces and indention
    SetTabWidth (4);
    SetUseTabs (false);
    SetTabIndents (true);
    SetBackSpaceUnIndents (true);
    SetIndent (g_CommonPrefs.indentEnable? 4: 0);

    // others
    SetViewEOL (g_CommonPrefs.displayEOLEnable);
    SetIndentationGuides (g_CommonPrefs.indentGuideEnable);
    SetEdgeColumn (80);
    SetEdgeMode (g_CommonPrefs.longLineOnEnable? wxSTC_EDGE_LINE: wxSTC_EDGE_NONE);
    SetViewWhiteSpace (g_CommonPrefs.whiteSpaceEnable?
                       wxSTC_WS_VISIBLEALWAYS: wxSTC_WS_INVISIBLE);
    SetOvertype (g_CommonPrefs.overTypeInitial);
    SetReadOnly (g_CommonPrefs.readOnlyInitial);
    SetWrapMode (g_CommonPrefs.wrapModeInitial?
                 wxSTC_WRAP_WORD: wxSTC_WRAP_NONE);

    return true;
}

void TASEditorTextCtrl::OnMarginClick(wxStyledTextEvent &event) {
  if (event.GetMargin() == 2) {
    int lineClick = LineFromPosition (event.GetPosition());
    int levelClick = GetFoldLevel (lineClick);
    if ((levelClick & wxSTC_FOLDLEVELHEADERFLAG) > 0) {
      ToggleFold (lineClick);
    }
  }
}

void TASEditorTextCtrl::OnEditRedo(wxCommandEvent &WXUNUSED(event)) {
  if (!CanRedo()) return;
  Redo ();
}

void TASEditorTextCtrl::OnEditUndo(wxCommandEvent &WXUNUSED(event)) {
  if (!CanUndo()) return;
  Undo ();
}

void TASEditorTextCtrl::OnEditClear(wxCommandEvent &WXUNUSED(event)) {
  if (GetReadOnly()) return;
  Clear();
}

void TASEditorTextCtrl::OnEditCut(wxCommandEvent &WXUNUSED(event)) {
  if (GetReadOnly() || (GetSelectionEnd()-GetSelectionStart() <= 0)) return;
  Cut ();
}

void TASEditorTextCtrl::OnEditCopy(wxCommandEvent &WXUNUSED(event)) {
  if (GetSelectionEnd()-GetSelectionStart() <= 0) return;
  Copy ();
}

void TASEditorTextCtrl::OnEditPaste(wxCommandEvent &WXUNUSED(event)) {
  if (!CanPaste()) return;
  Paste ();
}

TASEditorTextCtrl* TASEditorFrame::GetCurrentTextCtrl()
{
  return (TASEditorTextCtrl*)m_pEditorFrameNB->GetPage(GetCurrentTab());
}

void TASEditorFrame::OnFind(wxCommandEvent &WXUNUSED(event)) {
  m_findData->SetFlags(wxFR_DOWN);

  if (m_dlgFind)  delete m_dlgFind;
  m_dlgFind = new wxFindReplaceDialog(this, m_findData, _T("Find dialog"), wxFR_NOWHOLEWORD);
  m_dlgFind->Show(true);
}

void TASEditorFrame::OnFindPrev(wxCommandEvent& WXUNUSED(event)) {
  m_findData->SetFlags(m_findData->GetFlags() - (m_findData->GetFlags() & wxFR_DOWN));
  FindNext();
}

void TASEditorFrame::OnFindNext(wxCommandEvent& WXUNUSED(event)) {
  m_findData->SetFlags(m_findData->GetFlags() | wxFR_DOWN);
  FindNext();
}

void TASEditorFrame::FindNext() {
  wxString findStr = m_findData->GetFindString();

  if (findStr.Length() > 0) {
    int flags   = m_findData->GetFlags();
    int currPos = GetCurrentTextCtrl()->GetCurrentPos();

    int docEnd = GetCurrentTextCtrl()->GetLength();
    int startPos = -1;
    if (flags & wxFR_DOWN) {
      startPos = GetCurrentTextCtrl()->FindText(currPos, docEnd, findStr, flags);
    } else {
      currPos = GetCurrentTextCtrl()->GetSelectionStart();
      currPos = (currPos > 0) ? (currPos - 1) : currPos;
      startPos = GetCurrentTextCtrl()->FindText(currPos-1, 0, findStr, flags);
    }
    if (startPos >= 0) {
      GetCurrentTextCtrl()->SetSelectionStart(startPos);
      GetCurrentTextCtrl()->SetSelectionEnd(startPos + findStr.Length());
      GetCurrentTextCtrl()->EnsureCaretVisible();
    } else if (currPos > 0) {
      if (flags & wxFR_DOWN) {
        wxLogMessage(_T("Search wrapping and beginning at beginning of document."));
        startPos = GetCurrentTextCtrl()->FindText(0, currPos, findStr, flags);
      } else {
        wxLogMessage(_T("Search wrapping and beginning at end of document."));
        startPos = GetCurrentTextCtrl()->FindText(docEnd, currPos, findStr, flags);
      }
      if (startPos >= 0) {
        GetCurrentTextCtrl()->SetSelectionStart(startPos);
        GetCurrentTextCtrl()->SetSelectionEnd(startPos + findStr.Length());
        GetCurrentTextCtrl()->EnsureCaretVisible();
      } else {
        wxLogMessage(_T("Search text '%s' not found in document."), findStr.c_str());
      }
    }
  }
}

void TASEditorFrame::OnReplace(wxCommandEvent &WXUNUSED(event)) {
  m_findData->SetFlags(wxFR_DOWN);

  if (m_dlgReplace) delete m_dlgReplace;
  m_dlgReplace = new wxFindReplaceDialog(this, m_findData, _T("Find and replace dialog"), wxFR_REPLACEDIALOG);
  m_dlgReplace->Show(true);
}

void TASEditorFrame::OnReplaceNext(wxCommandEvent &WXUNUSED(event)) {
}

void TASEditorFrame::OnBookmarkAdd(wxCommandEvent &WXUNUSED(event))
{
  g_iMarkerCounter = GetCurrentTextCtrl()->MarkerAdd(GetCurrentTextCtrl()->GetCurrentLine(), 1);
}

void TASEditorFrame::OnBookmarkRem(wxCommandEvent &WXUNUSED(event))
{
  GetCurrentTextCtrl()->MarkerDelete(GetCurrentTextCtrl()->GetCurrentLine(), 1);
}

void TASEditorFrame::OnNextBookmark(wxCommandEvent &WXUNUSED(event))
{
  int currLine = GetCurrentTextCtrl()->GetCurrentLine();
  GetCurrentTextCtrl()->GotoLine(GetCurrentTextCtrl()->MarkerNext(GetCurrentTextCtrl()->GetCurrentLine()+1, 2));
  if (GetCurrentTextCtrl()->MarkerGet(GetCurrentTextCtrl()->GetCurrentLine()) == 0) {
    GetCurrentTextCtrl()->GotoLine(GetCurrentTextCtrl()->MarkerNext(GetCurrentTextCtrl()->GetCurrentLine()+1, 2));
    if (GetCurrentTextCtrl()->MarkerGet(GetCurrentTextCtrl()->GetCurrentLine()) == 0) {
      GetCurrentTextCtrl()->GotoLine(currLine);
    }
  }
}

void TASEditorFrame::OnPrevBookmark(wxCommandEvent &WXUNUSED(event))
{
  int currLine = GetCurrentTextCtrl()->GetCurrentLine();
  GetCurrentTextCtrl()->GotoLine(GetCurrentTextCtrl()->MarkerPrevious(GetCurrentTextCtrl()->GetCurrentLine()-1, 2));
  if ((GetCurrentTextCtrl()->MarkerGet(GetCurrentTextCtrl()->GetCurrentLine()) == 0) &&
      (GetCurrentTextCtrl()->GetCurrentLine() == 0))
  {
    GetCurrentTextCtrl()->DocumentEnd();
    GetCurrentTextCtrl()->Home();
    GetCurrentTextCtrl()->GotoLine(GetCurrentTextCtrl()->MarkerPrevious(GetCurrentTextCtrl()->GetCurrentLine(), 2));
    if (GetCurrentTextCtrl()->MarkerGet(GetCurrentTextCtrl()->GetCurrentLine()) == 0) {
      GetCurrentTextCtrl()->GotoLine(currLine);
    }
  }
}

void TASEditorTextCtrl::OnCommentSelection(wxCommandEvent &WXUNUSED(event)) {
  wxString commentStr = _T("//");

  int startPos, endPos;
  GetSelection(&startPos, &endPos);
  SetCurrentPos(startPos);

  while (startPos <= endPos) {
    AddText(commentStr);
    LineDown();
    CmdKeyExecute(wxSTC_CMD_HOME);
    startPos = GetCurrentPos();
    endPos += 2;
  }
}

void TASEditorTextCtrl::OnUncommentSelection(wxCommandEvent &WXUNUSED(event)) {
  wxString uncommentStr = _T("");
  
  int startPos, endPos;
  GetSelection(&startPos, &endPos);
  SetCurrentPos(startPos);
  
  while (startPos <= endPos) {
    int currChar = GetCharAt(startPos);
    int startLine = LineFromPosition(startPos);
    int currLine = startLine;
    while ((currChar < 33 || currChar > 126) && (currLine == startLine)) {  // 47 == '/'
      WordRight();
      startPos = GetCurrentPos();
      currChar = GetCharAt(startPos);
    }

    if (currChar == 47 && GetCharAt(startPos+1) == 47) {
      SetSelection(startPos, (startPos+2));
      ReplaceSelection(uncommentStr);
      endPos -= 2;
    }

    LineDown();
    CmdKeyExecute(wxSTC_CMD_HOME);
    startPos = GetCurrentPos();
  }
}


void TASEditorTextCtrl::OnBraceMatch(wxCommandEvent &WXUNUSED(event)) {
  int startPos, endPos;
  GetSelection(&startPos, &endPos);
  //  Walk forward through selection until a brace is encountered.
  //  Braces can be:  '{' (123), '(' (40), '[' (91), or '<' (60)
  int currChar = 0;
  while ((startPos != (endPos+1)) && (currChar != 40) && (currChar != 60) &&
         (currChar != 91) && (currChar != 123)) {
    currChar = GetCharAt(startPos++);
  }

  if (currChar == 40 || currChar == 60 || currChar == 91 || currChar == 123) {
    startPos -= 1;
    int max = BraceMatch(startPos);
    if (max > (startPos+1)) {
      BraceHighlight(startPos+1, max);
      SetSelection(startPos, max+1);
    }else{
      BraceBadLight(startPos);
    }
  }
}

void TASEditorTextCtrl::OnEditIndentInc(wxCommandEvent &WXUNUSED(event)) {
  CmdKeyExecute(wxSTC_CMD_TAB);
}

void TASEditorTextCtrl::OnEditIndentRed(wxCommandEvent &WXUNUSED(event)) {
  CmdKeyExecute(wxSTC_CMD_BACKTAB);
}

void TASEditorTextCtrl::OnEditSelectAll(wxCommandEvent &WXUNUSED(event)) {
  SetSelection(0, GetTextLength());
}

void TASEditorTextCtrl::OnEditSelectLine(wxCommandEvent &WXUNUSED(event)) {
  int lineStart = PositionFromLine (GetCurrentLine());
  int lineEnd = PositionFromLine (GetCurrentLine() + 1);
  SetSelection(lineStart, lineEnd);
}

void TASEditorTextCtrl::OnKey(wxKeyEvent& event)
{
  if ((event.GetModifiers() == wxMOD_CONTROL) && (event.GetKeyCode() == WXK_TAB)) {
    m_parentFrame->m_pEditorFrameNB->SetSelection((m_parentFrame->m_pEditorFrameNB->GetSelection() + 1) % m_parentFrame->m_pEditorFrameNB->GetPageCount());
  } else if ((event.GetModifiers() == (wxMOD_CONTROL|wxMOD_SHIFT)) && (event.GetKeyCode() == WXK_TAB)) {
    if (m_parentFrame->m_pEditorFrameNB->GetSelection() == 0) {
      m_parentFrame->m_pEditorFrameNB->SetSelection(m_parentFrame->m_pEditorFrameNB->GetPageCount() - 1);
    } else {
      m_parentFrame->m_pEditorFrameNB->SetSelection((m_parentFrame->m_pEditorFrameNB->GetSelection() - 1) % m_parentFrame->m_pEditorFrameNB->GetPageCount());
    }
  } else {
    event.Skip();
  }
}
*/

TASEditorFrame::TASEditorFrame(wxWindowID id, TASApp* parent, wxPoint& ptLoc, wxSize& szSize) 
                : TASFrameBase(id, parent, _T("TAS Editor Window"), ptLoc, szSize, wxDEFAULT_FRAME_STYLE | wxNO_FULL_REPAINT_ON_RESIZE)
{
  m_pParentFrame      = parent->m_pSimFrame;
  m_dlgFind           = NULL;
  m_dlgReplace        = NULL;
  m_pFileMenu         = NULL;
  m_pEditMenu         = NULL;
  m_pViewMenu         = NULL;
  m_pTilesetMenu      = NULL;
  m_findData          = new wxFindReplaceData();
//  m_ctrlEditorText    = NULL;
//  m_ctrlAnalyzerText  = NULL;
  m_pEditorFrameNB    = NULL;
  m_pMenuBar          = NULL;
  m_pTTWin            = NULL;

  SetTitle();

  // set the frame icon
//MJP: need to set icon for all platforms
#ifdef __WXMSW__
  wxIcon icon = wxXmlResource::Get()->LoadIcon(_T("TAS.ico"));
#elif defined(__WXGTK__) || defined(__WXMOTIF__) || defined(__WXMAC__)
  wxIcon icon = wxXmlResource::Get()->LoadIcon(_T("TAS.xpm"));
#endif
  SetIcon(icon);

  //  Create the menu bar, then create and attach each menu to it
  m_pMenuBar = new wxMenuBar();

  //  Create all menus
  SetFileMenu();
  SetEditMenu();

  m_pViewMenu = new wxMenu(_T(""), wxMENU_TEAROFF);
  m_pViewMenu->AppendCheckItem(Menu_View_ToggleShowMessages, _("Show &messages window\tCtrl-O"), _("Show/hide messages window"));
  m_pViewMenu->Check(Menu_View_ToggleShowMessages, true);
  m_pViewMenu->AppendCheckItem(Menu_View_ToggleShowTTEditor, _("Show tile type editor window"), _("Show/hide tile type editor window"));
  m_pViewMenu->Check(Menu_View_ToggleShowTTEditor, true);
  m_pViewMenu->AppendSeparator();
  m_pViewMenu->AppendCheckItem(Menu_View_ToggleShowSymbols, _T("Display tile labels"));
  m_pViewMenu->Check(Menu_View_ToggleShowSymbols, TAS_MASK_ISON(g_pContext->m_maskEditorTileDisplay, TAS_MASK_DISPLABEL));
  m_pViewMenu->AppendCheckItem(Menu_View_ToggleShowTileNames, _T("Display tile names"));
  m_pViewMenu->Check(Menu_View_ToggleShowTileNames, TAS_MASK_ISON(g_pContext->m_maskEditorTileDisplay, TAS_MASK_DISPNAME));
  m_pViewMenu->AppendCheckItem(Menu_View_ToggleShowTileGlues, _T("Display tile glues"));
  m_pViewMenu->Check(Menu_View_ToggleShowTileGlues, TAS_MASK_ISON(g_pContext->m_maskEditorTileDisplay, TAS_MASK_DISPGLUES));

  m_pMenuBar->Append(m_pViewMenu, _T("Appearance"));

  SetTilesetMenu();

  wxMenu* pHelpMenu = new wxMenu(_T(""), wxMENU_TEAROFF);
  pHelpMenu->Append(Menu_Help, _T("Help\tF1"), _T("Display help documentation"));
  m_pMenuBar->Append(pHelpMenu, _T("Help"));

  m_mgr.SetManagedWindow(this);

  // attach the menu bar to the frame
  SetMenuBar(m_pMenuBar);

  m_pEditorFrameNB = new wxAuiNotebook(this, wxID_ANY, wxDefaultPosition, wxSize(300, 400), wxAUI_NB_TOP | wxAUI_NB_TAB_SPLIT | wxAUI_NB_TAB_MOVE | wxAUI_NB_SCROLL_BUTTONS);

  if (m_pParentFrame->GetTASDimMode() == TAS_3D_MODE)
  {
    TASTiletypePanel* pep = new TASTiletypePanel();
    pep->m_fn = g_pContext->m_fnTilesetFile;
    pep->m_pEditor = new TASTileEditor(m_pEditorFrameNB, this, -1, wxDefaultPosition, wxSize(300, 400));
    m_pEditorFrameNB->AddPage(pep->m_pEditor, _T("Tiletype editor"));
    m_tileEditorPanels.push_back(pep);
  } else {
    TASTiletypePanel* pep = new TASTiletypePanel();
    pep->m_fn = g_pContext->m_fnTilesetFile;
    pep->m_pEditor = new TASTileEditor(m_pEditorFrameNB, this, -1, wxDefaultPosition, wxSize(300, 400));
    m_pEditorFrameNB->AddPage(pep->m_pEditor, _T("Tiletype editor"));
    m_tileEditorPanels.push_back(pep);
  }

  m_mgr.AddPane(m_pEditorFrameNB, wxAuiPaneInfo().Name(wxT("topPane")).Row(1).Show().Top().
                                CloseButton(false).CaptionVisible(false).CentrePane());

  m_pMessageTabs = new TASNotebookWin(this, wxID_ANY, &m_pViewMenu, Menu_View_ToggleShowMessages, wxDefaultPosition, wxSize(300, 150), wxAUI_NB_TOP | wxAUI_NB_TAB_SPLIT | wxAUI_NB_TAB_MOVE | wxAUI_NB_SCROLL_BUTTONS);

  m_ctrlOutputMsgs = new wxTextCtrl(m_pMessageTabs, -1, _T(""), wxDefaultPosition, wxDefaultSize,
                                    wxTE_MULTILINE | wxTE_READONLY);

  m_ctrlDebugMsgs = new wxTextCtrl(m_pMessageTabs, -1, _T(""), wxDefaultPosition, wxDefaultSize,
                                   wxTE_MULTILINE | wxTE_READONLY);

  m_pMessageTabs->AddPage(m_ctrlOutputMsgs, wxT("Output"));
  m_pMessageTabs->AddPage(m_ctrlDebugMsgs,  wxT("Debug"));

  m_mgr.AddPane(m_pMessageTabs, wxAuiPaneInfo().Name(wxT("messagesPane")).Caption(wxT("Messages")).Row(2).Show().Bottom());

  m_pTTWin = new TASTileTypeWin(this);

  m_mgr.AddPane(m_pTTWin, wxAuiPaneInfo().Name(wxT("tileTypeWin")).Caption(_T("Tile Type Definition")).Show().
                                     Left().CloseButton(true).CaptionVisible(true).BestSize(150, 50));

  if (g_pContext->m_sEditorFramePersp.Len() > 0)
  {
    m_mgr.LoadPerspective(g_pContext->m_sEditorFramePersp, false);
  }

  m_mgr.Update();
}

TASEditorFrame::~TASEditorFrame()
{
  m_helpController.Quit();
  m_mgr.UnInit();
  if (m_findData) {
    delete m_findData;
  }
  while (m_pEditorFrameNB->GetPageCount() > 0) {
    m_pEditorFrameNB->RemovePage(0);
  }
  while (m_tileEditorPanels.size()) {
    delete m_tileEditorPanels[m_tileEditorPanels.size()-1];
    m_tileEditorPanels.pop_back();
  }
}

//  Create 'file' menu targeted for current mode and selected tab
void TASEditorFrame::SetFileMenu()
{
  if (m_pFileMenu) {
    m_pMenuBar->Remove(0);
    delete m_pFileMenu;
  }

  m_pFileMenu = new wxMenu(_T(""), wxMENU_TEAROFF);
  m_pFileMenu->Append(Menu_File_NewTileset,              _T("New tile set\tCtrl-N"),      _T("Create a new tile set"));
  m_pFileMenu->AppendSeparator();
  m_pFileMenu->Append(Menu_File_LoadTileset,             _T("Load tile set...\tCtrl-T"),  _T("Load tile set definition file"));
  m_pFileMenu->Append(Menu_File_ReloadTileset,           _T("Reload tile set\tCtrl-R"),   _T("Reload tile set definition file"));
  m_pFileMenu->AppendSeparator();
  m_pFileMenu->Append(Menu_File_SaveTileset,             _T("Save tile set\tCtrl-S"),     _T("Save the current tile set"));
  m_pFileMenu->Append(Menu_File_SaveTilesetAs,           _T("Save tile set as..."),       _T("Save the current tile set as..."));
  m_pFileMenu->AppendSeparator();
  m_pFileMenu->Append(Menu_File_TilesetImages,           _T("Save tile set images..."),   _T("Save images of each tile type..."));
  m_pFileMenu->AppendSeparator();
  m_pFileMenu->Append(Menu_File_Quit,                    _T("Quit\tCtrl-Q"),              _T("Exit ISU TAS"));
  m_pMenuBar->Insert(0, m_pFileMenu, _T("File"));
}

void TASEditorFrame::SetEditMenu()
{
  if ((m_pEditMenu) && (m_pMenuBar->FindMenu(_T("Edit")) != wxNOT_FOUND)) {
    m_pMenuBar->Remove(m_pMenuBar->FindMenu(_T("Edit")));
    delete m_pEditMenu;
  }

  m_pEditMenu = new wxMenu(_T(""), wxMENU_TEAROFF);
  TASTileEditor::BuildEditMenu(m_pEditMenu);

  m_pMenuBar->Insert(1, m_pEditMenu, _T("Edit"));
}

void TASEditorFrame::SetTilesetMenu()
{
  if (m_pTilesetMenu) {
    int index = m_pMenuBar->FindMenu(_T("Tile set"));
    if (index != wxNOT_FOUND) {
      m_pMenuBar->Remove(index);
    }
    delete m_pTilesetMenu;
  }
  m_pTilesetMenu = new wxMenu(_T(""), wxMENU_TEAROFF);
  m_pTilesetMenu->Append(Menu_Tileset_IntoSimulator,  _T("Load tile set into simulator\tCtrl-L"), _T("Load the current tile set into the simulator"));
  m_pMenuBar->Append(m_pTilesetMenu, _T("Tile set"));
}

void TASEditorFrame::OnToggleShowMessages(wxCommandEvent& WXUNUSED(event))
{
  if (!m_pMessageTabs) {
    return;
  }
  if (m_mgr.GetPane(m_pMessageTabs).IsShown()) {
    m_mgr.GetPane(m_pMessageTabs).Hide();
  } else {
    m_mgr.GetPane(m_pMessageTabs).Show();
  }
  m_mgr.Update();
}

void TASEditorFrame::OnToggleShowTTEditor(wxCommandEvent& WXUNUSED(event))
{
  if (!m_pTTWin) {
    return;
  }
  if (m_mgr.GetPane(m_pTTWin).IsShown()) {
    m_mgr.GetPane(m_pTTWin).Hide();
  } else {
    m_mgr.GetPane(m_pTTWin).Show();
  }
  m_mgr.Update();
}

void TASEditorFrame::OnToggleShowSymbols(wxCommandEvent& WXUNUSED(event))
{
  g_pContext->m_maskEditorTileDisplay = TAS_MASK_ISON(g_pContext->m_maskEditorTileDisplay, TAS_MASK_DISPLABEL) ?
                                          g_pContext->m_maskEditorTileDisplay - TAS_MASK_DISPLABEL :
                                          g_pContext->m_maskEditorTileDisplay + TAS_MASK_DISPLABEL;
  g_pEditorTileset->InvalidateBitmaps();
  Refresh();
}

void TASEditorFrame::OnToggleShowTileNames(wxCommandEvent& WXUNUSED(event))
{
  g_pContext->m_maskEditorTileDisplay = TAS_MASK_ISON(g_pContext->m_maskEditorTileDisplay, TAS_MASK_DISPNAME) ?
                                          g_pContext->m_maskEditorTileDisplay - TAS_MASK_DISPNAME :
                                          g_pContext->m_maskEditorTileDisplay + TAS_MASK_DISPNAME;
  g_pEditorTileset->InvalidateBitmaps();
  Refresh();
}

void TASEditorFrame::OnToggleShowTileGlues(wxCommandEvent& WXUNUSED(event))
{
  g_pContext->m_maskEditorTileDisplay = TAS_MASK_ISON(g_pContext->m_maskEditorTileDisplay, TAS_MASK_DISPGLUES) ?
                                          g_pContext->m_maskEditorTileDisplay - TAS_MASK_DISPGLUES :
                                          g_pContext->m_maskEditorTileDisplay + TAS_MASK_DISPGLUES;
  g_pEditorTileset->InvalidateBitmaps();
  Refresh();
}

void TASEditorFrame::SetDimMode()
{
  SetEditMenu();

  m_mgr.DetachPane(m_pTTWin);
  delete m_pTTWin;
  m_pTTWin = new TASTileTypeWin(this);
  m_mgr.AddPane(m_pTTWin, wxAuiPaneInfo().Name(wxT("tileTypeWin")).Caption(_T("Tile Type Definition")).Show().
                Left().CloseButton(true).CaptionVisible(true).BestSize(150, 50));

  m_mgr.Update();
  if (m_pEditorFrameNB) {
    TASTileEditor* pCurrEd = (TASTileEditor*)m_pEditorFrameNB->GetPage(GetCurrentTab());
    if (pCurrEd) {
      wxSizeEvent evt;
      pCurrEd->OnSize(evt);
    }
  }
  Refresh();
}

void TASEditorFrame::ProcessUpdatedTileset()
{
  SetTitle();
//  Update();
  Refresh();
  wxString msg;
  msg << _T("Tile set '") << g_pContext->m_fnTilesetFile.GetFullName().c_str() << _T("' successfully loaded into editor with ") << g_pEditorTileset->size() << _T(" tile type definitions.\n");
  if ((g_pContext->m_fnTilesetFile.GetFullName() == DefaultTilesetFileName) ||
      (g_pContext->m_fnTilesetFile.GetFullName() == _T("")))
  {
    msg = wxString::Format(_T("New tile set created in editor with %d tile type definitions.\n"), g_pEditorTileset->size());
  }
  g_pTASApp->SetEditorOutputText(msg, true);

  g_pContext->m_iTASTileConcMode = TAS_TILECONC_MODE_OFF;
  for (size_t i=0; i<g_pEditorTileset->size(); i++) {
    if (g_pEditorTileset->at(i)->m_concentration > 1) {
      g_pContext->m_iTASTileConcMode = TAS_TILECONC_MODE_ON;
      break;
    }
  }
  if (g_pTASApp && g_pTASApp->m_pSimFrame) {
    g_pTASApp->m_pSimFrame->SetUseTileConcCheck();
  }

  if (m_pTTWin) {
    if (m_pTTWin->GetCurrentDimMode() != g_pContext->m_iTASDimMode) {
      SetDimMode();
    }
    m_pTTWin->SetConcentrationVisibility();
    m_pTTWin->PopulateLists();
  }
  g_pEditorTileset->SetDiskMod(false);
  g_pEditorTileset->SetSimMod(true);

  if (m_pEditorFrameNB) {
    TASTileEditor* pCurrEd = (TASTileEditor*)m_pEditorFrameNB->GetPage(GetCurrentTab());
    if (pCurrEd) {
      wxSizeEvent evt;
      pCurrEd->OnSize(evt);
    }
  }
}

void TASEditorFrame::OnNewTileset(wxCommandEvent& WXUNUSED(event))
{
  bool bDoIt = true;
  if (g_pEditorTileset->GetDiskMod()) {
    wxMessageDialog dlg(this, _T("Are you sure you want to create a new tile set file and\nabandon any unsaved changes you have made to this one?"), _T("Confirm abandon changes"), wxOK|wxCANCEL|wxCENTRE);
    if (dlg.ShowModal() != wxID_OK) {
      bDoIt = false;
    }
  }
  if (bDoIt) {
    g_pTASApp->NewTileset();
  }
}

void TASEditorFrame::OnLoadTileset(wxCommandEvent& WXUNUSED(event))
{
  if (g_pEditorTileset->GetDiskMod()) {
    wxMessageDialog dlg(this, _T("Are you sure you want to load a new tile set file and\nabandon any unsaved changes you have made?"), _T("Confirm abandon changes"), wxOK|wxCANCEL|wxCENTRE);
    if (dlg.ShowModal() != wxID_OK) {
      return;
    }
  }
  bool bDoLoad = false;
  wxString sFn;
  if (g_pContext->m_fnTilesetFile.GetFullName().Len() > 0) {
    sFn = g_pContext->m_fnTilesetFile.GetFullName();
  }
  wxFileDialog fDlg(this, _T("Load tile set from..."), _T(""),
                    sFn, _T("TDS files (*.tds)|*.tds"), wxFD_OPEN | wxFD_FILE_MUST_EXIST | wxFD_CHANGE_DIR);
  fDlg.SetDirectory(g_pContext->m_fnTilesetFile.IsOk() 
                    ? g_pContext->m_fnTilesetFile.GetPath(wxPATH_GET_VOLUME)
                    : wxGetCwd());
  while (!bDoLoad) {
    if (fDlg.ShowModal() == wxID_OK) {
      if (wxFileExists(fDlg.GetPath())) {
        wxString sPath = fDlg.GetPath();
        if (g_pTASApp->LoadTilesetIntoEditor(sPath)) {

        } else {
          wxString errMsg = wxString::Format(_T("There has been an error opening the tile set file:\n'%s'.\nPlease check to make sure the file exists and permissions allow this."), fDlg.GetPath().c_str());
          wxMessageBox(errMsg, _T("Error loading tile set"), wxICON_ERROR|wxOK);
        }
        bDoLoad = true;
      } else {
        wxString errMsg = wxString::Format(_T("There has been an error opening the tile set file:\n'%s'.\nPlease check to make sure the file exists and permissions allow this."), fDlg.GetPath().c_str());
        wxMessageBox(errMsg, _T("Error loading tile set"), wxICON_ERROR|wxOK);
      }
    } else {
      break;
    }
  }
}

void TASEditorFrame::OnReloadTileset(wxCommandEvent& WXUNUSED(event))
{
  if (g_pContext->IsTilesetFileOK()) {
    if (g_pEditorTileset->GetDiskMod()) {
      wxMessageDialog dlg(this, _T("Are you sure you want to reload the tile set file and\nabandon any unsaved changes you have made?"), _T("Confirm abandon changes"), wxOK|wxCANCEL|wxCENTRE);
      if (dlg.ShowModal() != wxID_OK) {
        return;
      }
    }
    if (g_pTASApp->ReloadTilesetIntoEditor()) {

    } else {
      wxString errMsg = wxString::Format(_T("There has been an error reloading the tile set. Please\ncheck to make sure the file exists and permissions allow this."));
      wxMessageBox(errMsg, _T("Error reloading tile set"), wxICON_ERROR|wxOK);
    }
  } else {
    wxMessageBox(_T("There is no current tile set file to reload!"), _T("Error reloading tile set"), wxICON_ERROR|wxOK);
  }
}

void TASEditorFrame::OnSaveTileset(wxCommandEvent& event)
{
  if (g_pContext->IsTilesetFileOK()) {
    if (!g_pTASApp->SaveTileset()) {
      wxString errMsg = wxString::Format(_T("There has been an error saving the tile set. Please\ncheck to make sure the file permissions allow this."));
      wxMessageBox(errMsg, _T("Error saving tile set"), wxICON_ERROR|wxOK);
    } else {
      g_pEditorTileset->SetDiskMod(false);
    }
  } else {
    OnSaveTilesetAs(event);
  }
}

void TASEditorFrame::OnSaveTilesetAs(wxCommandEvent& WXUNUSED(event))
{
  bool bDoLoad = false;
  wxString sFn;
  if (g_pContext->m_fnTilesetFile.GetFullName().Len() > 0) {
    sFn = g_pContext->m_fnTilesetFile.GetFullName();
  }
  wxFileDialog fDlg(this, _T("Save tile set as..."), _T(""),
                    sFn, _T("TDS files (*.tds)|*.tds"), wxFD_SAVE | wxFD_CHANGE_DIR);
  fDlg.SetDirectory(g_pContext->m_fnTilesetFile.IsOk() 
                    ? g_pContext->m_fnTilesetFile.GetPath(wxPATH_GET_VOLUME)
                    : wxGetCwd());
  while (!bDoLoad) {
    if (fDlg.ShowModal() == wxID_OK) {
      wxString sPath = fDlg.GetPath();
      if (!g_pTASApp->SaveTilesetAs(sPath)) {
        wxString errMsg = wxString::Format(_T("There has been an error saving the tile set to file:\n'%s'.\nPlease check to make sure the file exists and permissions allow this."), fDlg.GetPath().c_str());
        wxMessageBox(errMsg, _T("Error saving tile set"), wxICON_ERROR|wxOK);
      } else {
        SetTitle();
        g_pEditorTileset->SetDiskMod(false);
      }
      bDoLoad = true;
    } else {
      break;
    }
  }
}

void TASEditorFrame::OnTilesetToSimulator(wxCommandEvent& WXUNUSED(event))
{
  wxMessageDialog confDlg(this, _T("Are you sure you want to replace the tile set in the\nsimulator and reset the current assembly?"),
                          _T("Confirm replace tile set and reset assembly"), wxOK|wxCANCEL|wxCENTRE);
  if (confDlg.ShowModal() == wxID_OK) {
    if (g_pTASApp->LoadTilesetIntoSimulator()) {
      g_pTASApp->ResetAssembly();
      g_pTASApp->m_pSimFrame->DoRefresh();
      g_pEditorTileset->SetSimMod(false);
      g_pTASApp->m_pSimFrame->m_panel->ScrollToCenter(g_pTileSpace->m_seed.m_coordsLeftmost, false);
    }
  }
}

BEGIN_EVENT_TABLE(TASTileImageDialog, wxDialog)
  EVT_BUTTON(wxID_OK,                   TASTileImageDialog::OnOk)
  EVT_BUTTON(Dialog_Button_ID_IMAGEDIR, TASTileImageDialog::OnDirectoryButton)
  EVT_TEXT(Dialog_Text_ID_IMAGEDIR, TASTileImageDialog::OnText)
END_EVENT_TABLE()

TASTileImageDialog::TASTileImageDialog(wxWindow* parent, int type)
                       : wxDialog(parent, wxID_ANY, _T(""), wxDefaultPosition)
{
  m_iDlgType  = type;
  m_pDirText  = NULL;
  m_sDir      = wxGetCwd();

  wxBoxSizer* mainSizer = new wxBoxSizer(wxVERTICAL);

  wxString sTitle;
  wxString sTop;
  wxString sButton;
  if (m_iDlgType == TAS_IMAGE_DLG_TILETYPES) {
    sTitle = _T("Save images of all tiletypes");
    sTop += _T("Use this dialog to save images of each tiletype.  Select the type of images,\n");
    sTop += _T("their size, and the directory in which the image files should be saved.");
    sButton = _T("Directory");
  } else {
    sTitle = _T("Save image of tilespace");
    sTop += _T("Use this dialog to save an image of the tilespace.  Select the type of image,\n");
    sTop += _T("its size, and the file name for the output image file.");
    sButton = _T("Image file");
  }
  SetTitle(sTitle);

  wxStaticText* textTop = new wxStaticText(this, wxID_ANY, sTop);
  mainSizer->Add(textTop, 0, wxTOP | wxBOTTOM | wxLEFT | wxRIGHT, 10);

  mainSizer->AddSpacer(10);

  wxBoxSizer* dirSizer = new wxBoxSizer(wxHORIZONTAL);
  wxButton* dirButton = new wxButton(this, Dialog_Button_ID_IMAGEDIR, sButton);
  dirSizer->Add(dirButton);

  m_pDirText = new wxTextCtrl(this, Dialog_Text_ID_IMAGEDIR, m_sDir, wxDefaultPosition, wxSize(270, 20));
  dirSizer->Add(m_pDirText, 0, wxLEFT, 5);

  mainSizer->Add(dirSizer, 0, wxTOP | wxBOTTOM | wxLEFT | wxRIGHT, 10);

  wxString arrCh[]  = {_("Windows BMP"), _("GIF"), _("X bitmap"), _("XPM bitmap"), _("JPEG"), _("TIF"), _("PNG"), _("PCX"), _("PNM"), _("SVG")};
  m_pRadioMode = new wxRadioBox(this, Dialog_Radio_ID_IMAGETYPE, _T("Image type"),
                                wxDefaultPosition, wxDefaultSize, 10, arrCh, 3, wxRA_SPECIFY_ROWS);
  m_pRadioMode->SetSelection(0);
  mainSizer->AddSpacer(5);
  mainSizer->Add(m_pRadioMode, 0, wxLEFT, 50);
  mainSizer->AddSpacer(5);

  wxBoxSizer* buttonSizer = new wxBoxSizer(wxHORIZONTAL);

  wxButton* buttonOk = new wxButton(this, wxID_OK, _T("OK"));
  buttonSizer->Add(buttonOk, 0, wxLEFT, 200);

  wxButton* buttonCancel = new wxButton(this, wxID_CANCEL, _T("Cancel"));
  buttonSizer->Add(buttonCancel);

  mainSizer->Add(buttonSizer, 0, wxTOP | wxBOTTOM | wxLEFT | wxRIGHT, 10);

  SetAutoLayout(TRUE);
  SetSizer(mainSizer);
  
  mainSizer->SetSizeHints(this);
  mainSizer->Fit(this);
}

void TASTileImageDialog::OnOk(wxCommandEvent& WXUNUSED(evt))
{
  m_iImgType   = wxBITMAP_TYPE_BMP;
  m_sExt       = _T("bmp");
  if (m_pRadioMode->GetSelection() == 1) {
    m_iImgType = wxBITMAP_TYPE_GIF;
    m_sExt     = _T("gif");
  } else if (m_pRadioMode->GetSelection() == 2) {
    m_iImgType = wxBITMAP_TYPE_XBM;
    m_sExt     = _T("xbm");
  } else if (m_pRadioMode->GetSelection() == 3) {
    m_iImgType = wxBITMAP_TYPE_XPM;
    m_sExt     = _T("xpm");
  } else if (m_pRadioMode->GetSelection() == 4) {
    m_iImgType = wxBITMAP_TYPE_JPEG;
    m_sExt     = _T("jpg");
  } else if (m_pRadioMode->GetSelection() == 5) {
    m_iImgType = wxBITMAP_TYPE_TIF;
    m_sExt     = _T("tif");
  } else if (m_pRadioMode->GetSelection() == 6) {
    m_iImgType = wxBITMAP_TYPE_PNG;
    m_sExt     = _T("png");
  } else if (m_pRadioMode->GetSelection() == 7) {
    m_iImgType = wxBITMAP_TYPE_PCX;
    m_sExt     = _T("pcx");
  } else if (m_pRadioMode->GetSelection() == 8) {
    m_iImgType = wxBITMAP_TYPE_PNM;
    m_sExt     = _T("pnm");
  } else if (m_pRadioMode->GetSelection() == 9) {
    m_iImgType = (wxBitmapType)TAS_IMAGE_TYPE_SVG;
    m_sExt     = _T("svg");
  }
  EndModal(wxID_OK);
}

void TASTileImageDialog::OnDirectoryButton(wxCommandEvent& WXUNUSED(evt))
{
  if (m_iDlgType == TAS_IMAGE_DLG_TILETYPES) {
    wxDirDialog ddlg(this, _T("Select a directory"), m_sDir);
    if (ddlg.ShowModal() == wxID_OK) {
      m_sDir = ddlg.GetPath();
      m_pDirText->SetValue(m_sDir);
    }
  } else {
    wxFileDialog fdlg(this, _T("Select a file"), m_sDir);
    if (fdlg.ShowModal() == wxID_OK) {
      m_sDir = fdlg.GetPath();
      m_pDirText->SetValue(m_sDir);
    }
  }
}

void TASTileImageDialog::OnText(wxCommandEvent& WXUNUSED(evt))
{
  if (m_pDirText) {
    m_sDir = m_pDirText->GetValue();
  }
}

void TASEditorFrame::OnSaveTilesetImages(wxCommandEvent& WXUNUSED(event))
{
  bool bSucceeded = false;

  TASTileImageDialog dlg(this, TAS_IMAGE_DLG_TILETYPES);

  if (dlg.ShowModal() == wxID_OK) {
    wxInitAllImageHandlers();

    if (!wxDirExists(dlg.m_sDir.c_str())) {
      if (!wxMkdir(dlg.m_sDir.c_str())) {
        wxString sMsg = wxString::Format(_T("Failed to create directory '%s', so images could not be saved"), dlg.m_sDir.c_str());
        wxMessageBox(sMsg, _T("Error saving image files"), wxICON_ERROR|wxOK);
        if (g_pTASApp && g_pTASApp->m_pEditorFrame) {
          sMsg += _T("\n");
          g_pTASApp->SetEditorOutputText(sMsg, true, false);
        }
        return;
      }
    }
    bSucceeded = true;

    for (size_t i=0; i<g_pEditorTileset->size(); i++) {
      wxString fn;
      if (g_pEditorTileset->at(i)->m_sName.Len() > 0) {
        fn = wxString::Format(_T("%s\\%s.%s"), dlg.m_sDir.c_str(), g_pEditorTileset->at(i)->m_sName.c_str(), dlg.m_sExt.c_str());
      } else {
        fn = wxString::Format(_T("%s\\%d.%s"), dlg.m_sDir.c_str(), i, dlg.m_sExt.c_str());
      }
      if (dlg.m_iImgType < TAS_IMAGE_TYPE_SVG) {
        wxBitmap& bm = g_pEditorTileset->at(i)->GetBitmap(g_tileSize);
        if (bm.IsOk()) {
          if (!bm.SaveFile(fn, dlg.m_iImgType)) {
            wxString sMsg;
            if (bSucceeded) {
              sMsg  = _T("Error saving tile type images to files. Please\n");
              sMsg += _T("see the Output message pane for details.");
              wxMessageBox(sMsg, _T("Error saving images"), wxICON_ERROR|wxOK);
              bSucceeded = false;
            }
            if (g_pTASApp && g_pTASApp->m_pEditorFrame) {
              sMsg = wxString::Format(_T("Failed to save image of tiletype '%s' to file '%s'\n"), g_pEditorTileset->at(i)->m_sName.c_str(), fn.c_str());
              g_pTASApp->SetEditorOutputText(sMsg, true, false);
            }
          }
        } else {
          wxString sMsg;
          if (bSucceeded) {
            sMsg  = _T("Error saving tile type images to files. Please\n");
            sMsg += _T("see the Output message pane for details.");
            wxMessageBox(sMsg, _T("Error saving images"), wxICON_ERROR|wxOK);
            bSucceeded = false;
          }
          if (g_pTASApp && g_pTASApp->m_pEditorFrame) {
            sMsg = wxString::Format(_T("Failed to get image of tiletype '%s'\n"), g_pEditorTileset->at(i)->m_sName.c_str());
            g_pTASApp->SetEditorOutputText(sMsg, true, false);
          }
        }
      } else {
        wxMessageBox(_T("Sorry, this is not currently implemented."), _T("Error saving images"), wxICON_ERROR|wxOK);
/*
        int ts = g_tileSize;
        //g_tileSize = 200;
        wxSVGFileDC svgDC(fn, g_tileSize, g_tileSize);
        wxPoint p(0, 0);
        g_pEditorTileset->at(i)->DrawSVG(&svgDC, p, 200, g_pContext->m_maskEditorTileDisplay);
        g_tileSize = ts;
        if (!svgDC.Ok()) {
          break;
        }
*/
      }
    }
  }
  if (bSucceeded) {
    if (g_pTASApp && g_pTASApp->m_pEditorFrame) {
      wxString sMsg = wxString::Format(_T("Successfully output images of %d tile types to directory '%s'\n"), g_pEditorTileset->size(), dlg.m_sDir.c_str());
      g_pTASApp->SetEditorOutputText(sMsg, true, false);
    }
  }
}

void TASEditorFrame::OnEdit(wxCommandEvent &event) {
	if(m_pEditorFrameNB->GetPage(GetCurrentTab())) {
		m_pEditorFrameNB->GetPage(GetCurrentTab())->ProcessWindowEvent(event);
	}
}

/*void TASEditorFrame::SetTabTitle(int tabNum)
{
  if (tabNum == -1) {
    tabNum = GetCurrentTab();
  }

  if (tabNum == 0) {
    wxString sTilesetTabTitle = wxString::Format(_T("Tileset: %s"), g_pContext->m_fnTilesetFile.GetFullName().c_str());
    if (m_ctrlEditorText->GetModify()) {
      sTilesetTabTitle += _T("*");
    }
    m_pEditorFrameNB->SetPageText(0, sTilesetTabTitle);
  } else {
    wxString sSeedTabTitle = wxString::Format(_T("Seed: %s"), g_pContext->m_fnTilespaceFile.GetFullName().c_str());
    if (m_ctrlAnalyzerText->GetModify()) {
      sSeedTabTitle += _T("*");
    }
    m_pEditorFrameNB->SetPageText(1, sSeedTabTitle);
  }
}*/

void TASEditorFrame::OnClose(wxCloseEvent& event)
{
  m_pParentFrame->OnClose(event);
}

void TASEditorFrame::SetTitle()
{
  wxString sTitle;
  sTitle = wxString::Format(_T("ISU TAS Tileset Editor: %s (%s)"), 
                            g_pContext->m_fnTilesetFile.GetFullName().c_str(),
                            g_pContext->m_fnTilesetFile.GetFullPath().c_str());
  wxFrame::SetTitle(sTitle);
}

// print event handlers
void TASEditorFrame::OnPrintSetup(wxCommandEvent &WXUNUSED(event)) {
#if wxUSE_PRINTING_ARCHITECTURE
/*
  (*g_pageSetupData) = * g_printData;
  wxPageSetupDialog pageSetupDialog(this, g_pageSetupData);
  pageSetupDialog.ShowModal();
  (*g_printData) = pageSetupDialog.GetPageSetupData().GetPrintData();
  (*g_pageSetupData) = pageSetupDialog.GetPageSetupData();
*/
#endif // wxUSE_PRINTING_ARCHITECTURE
}

void TASEditorFrame::OnPrintPreview(wxCommandEvent &WXUNUSED(event)) {
#if wxUSE_PRINTING_ARCHITECTURE
/*
  wxPrintDialogData printDialogData(*g_printData);
  wxPrintPreview *preview = new wxPrintPreview(new TASEditorTextPrint(GetCurrentTextCtrl()),
                                               new TASEditorTextPrint(GetCurrentTextCtrl()),
                                               &printDialogData);
  if (!preview->Ok()) {
    delete preview;
    wxMessageBox (_("There was a problem with previewing.\n\
                     Perhaps your current printer is not connected?"),
                  _("Previewing"), wxOK);
    return;
  }
  wxRect rect = DeterminePrintSize();
  wxPreviewFrame* frame = new wxPreviewFrame(preview, this, _("Print Preview"));
  frame->SetSize(rect);
  frame->Centre(wxBOTH);
  frame->Initialize();
  frame->Show(true);
*/
#endif // wxUSE_PRINTING_ARCHITECTURE
}

void TASEditorFrame::OnPrint(wxCommandEvent &WXUNUSED(event)) {
#if wxUSE_PRINTING_ARCHITECTURE
/*
  wxPrintDialogData printDialogData(*g_printData);
  wxPrinter printer(&printDialogData);
  TASEditorTextPrint printout(GetCurrentTextCtrl());
  if (!printer.Print(this, &printout, true)) {
    if (wxPrinter::GetLastError() == wxPRINTER_ERROR) {
    wxMessageBox (_("There was a problem with printing.\n\
                     Perhaps your current printer is not connected?"),
                  _("Previewing"), wxOK);
    return;
    }
  }
  (*g_printData) = printer.GetPrintDialogData().GetPrintData();
*/
#endif // wxUSE_PRINTING_ARCHITECTURE
}

wxRect TASEditorFrame::DeterminePrintSize() {

  wxSize scr = wxGetDisplaySize();
  
  // determine position and size (shifting 16 left and down)
  wxRect rect = GetRect();
  rect.x += 16;
  rect.y += 16;
  rect.width = wxMin(rect.width, (scr.x - rect.x));
  rect.height = wxMin(rect.height, (scr.x - rect.y));
  
  return rect;
}

/*
void TASEditorFrame::OnFindDialog(wxFindDialogEvent& event)
{
  wxEventType type = event.GetEventType();

  if (type == wxEVT_COMMAND_FIND || type == wxEVT_COMMAND_FIND_NEXT)
  {
    FindNext();
  }
  else if (type == wxEVT_COMMAND_FIND_REPLACE ||
           type == wxEVT_COMMAND_FIND_REPLACE_ALL)
  {
    wxLogMessage(wxT("Replace %s'%s' with '%s'"),
      type == wxEVT_COMMAND_FIND_REPLACE_ALL ? _T("all ") : wxT(""),
      event.GetFindString().c_str(),
      event.GetReplaceString().c_str());
  }
  else if (type == wxEVT_COMMAND_FIND_CLOSE)
  {
    wxFindReplaceDialog *dlg = event.GetDialog();

    if (dlg == m_dlgFind) {
      m_dlgFind = NULL;
    } else if (dlg == m_dlgReplace) {
      m_dlgReplace = NULL;
    } else {
      wxFAIL_MSG( _T("unexpected event") );
    }
    
    dlg->Destroy();
  }
  else
  {
    wxLogError(wxT("Unknown find dialog event!"));
  }
}
*/

void TASEditorFrame::OnTileEditorEditMenu(wxCommandEvent& event)
{
  TASTileEditor* pCurrEd = (TASTileEditor*)m_pEditorFrameNB->GetPage(GetCurrentTab());
  if (pCurrEd && pCurrEd->GetEventHandler()) {
    pCurrEd->GetEventHandler()->AddPendingEvent(event);
  }
}

/*
void TASEditorFrame::SetEditorCursorPosition(int pos)
{
  GetCurrentTextCtrl()->SetSelectionStart(pos);
  GetCurrentTextCtrl()->SetSelectionEnd(pos);
  GetCurrentTextCtrl()->EnsureCaretVisible();
}

bool TASEditorFrame::HasUnsavedWork()
{
  if (m_ctrlEditorText->GetModify() || m_ctrlAnalyzerText->GetModify()) {
    return true;
  }
  return false;
}*/

WX_DEFINE_OBJARRAY(TASTextEntryOptionArray);
WX_DEFINE_OBJARRAY(TASTextEntryDlgDataArray);

BEGIN_EVENT_TABLE(TASTextEntryDialog, wxDialog)
  EVT_BUTTON(wxID_OK, TASTextEntryDialog::OnOK)
END_EVENT_TABLE()

IMPLEMENT_CLASS(TASTextEntryDialog, wxDialog)


TASTextEntryDialog::TASTextEntryDialog(wxWindow *parent, const wxString& caption,
                                           const wxString& mainText,
                                           TASTextEntryOptionArray& tteoa,
                                           long style, const wxPoint& pos)
                       : wxDialog(parent, -1, caption, pos, wxDefaultSize,
                                  wxDEFAULT_DIALOG_STYLE)
{
  m_dialogStyle = style;
  wxBeginBusyCursor();
  wxBoxSizer* topsizer = new wxBoxSizer(wxVERTICAL);
  wxBoxSizer* textsizer = new wxBoxSizer(wxHORIZONTAL);
  textsizer->Add(CreateTextSizer(mainText), 0, wxALL, 10);
  topsizer->Add(textsizer, 0, wxALIGN_CENTER);
  
  for (size_t i=0; i < tteoa.GetCount(); i++) {
    wxBoxSizer* sizer = new wxBoxSizer(wxHORIZONTAL);
    sizer->Add(CreateTextSizer(tteoa[i].m_message), 0, wxALL, 10);
    wxTextCtrl* textCtrl = new wxTextCtrl(this, -1/*wxID_TEXT*/, tteoa[i].m_value,
                                          wxDefaultPosition, wxSize(tteoa[i].m_width, -1),
                                          style & ~wxTextEntryDialogStyle | wxTE_RIGHT);
    sizer->Add(textCtrl, 1, wxALIGN_CENTER, 15);
    topsizer->Add(sizer, 0, wxALIGN_CENTER);
    m_dataArray.Add(TASTextEntryDlgData(tteoa[i].m_value, textCtrl));
  }
  
#if wxUSE_STATLINE
  topsizer->Add(new wxStaticLine(this, -1), 0, wxEXPAND|wxLEFT|wxRIGHT|wxTOP, 10);
#endif
  
  topsizer->Add(CreateButtonSizer(style), 0, wxCENTRE|wxALL, 10);
  
  SetAutoLayout(TRUE);
  SetSizer(topsizer);
  
  topsizer->SetSizeHints(this);
  topsizer->Fit(this);
  
  Centre(wxBOTH);
  m_dataArray[0].m_pTextCtrl->SetFocus();
  
  wxEndBusyCursor();
}

void TASTextEntryDialog::OnOK(wxCommandEvent& WXUNUSED(event) )
{
  for (size_t i=0; i<m_dataArray.GetCount(); i++) {
    m_dataArray[i].m_value = m_dataArray[i].m_pTextCtrl->GetValue();
  }
  EndModal(wxID_OK);
}

wxString TASTextEntryDialog::GetValue(int valueNum) const {
  if (valueNum <= (int)m_dataArray.GetCount()) {
    return m_dataArray[valueNum].m_value;
  } else {
    return wxString(_T(""));
  }
}

#if wxUSE_PRINTING_ARCHITECTURE

//----------------------------------------------------------------------------
// TASEditorTextPrint
//----------------------------------------------------------------------------
/*
TASEditorTextPrint::TASEditorTextPrint(TASEditorTextCtrl* edit, wxChar* title)
                       : wxPrintout(title)
{
    m_edit = edit;
    m_printed = 0;

}

bool TASEditorTextPrint::OnPrintPage (int page) {

    wxDC *dc = GetDC();
    if (!dc) return false;

    // scale DC
    PrintScaling (dc);

    // print page
    if (page == 1) m_printed = 0;
    m_printed = m_edit->FormatRange (1, m_printed, m_edit->GetLength(),
                                     dc, dc, m_printRect, m_pageRect);

    return true;
}

bool TASEditorTextPrint::OnBeginDocument (int startPage, int endPage) {

    if (!wxPrintout::OnBeginDocument (startPage, endPage)) {
        return false;
    }

    return true;
}

void TASEditorTextPrint::GetPageInfo (int *minPage, int *maxPage, int *selPageFrom, int *selPageTo) {

    // initialize values
    *minPage = 0;
    *maxPage = 0;
    *selPageFrom = 0;
    *selPageTo = 0;

    // scale DC if possible
    wxDC *dc = GetDC();
    if (!dc) return;
    PrintScaling (dc);

    // get print page informations and convert to printer pixels
    wxSize ppiScr;
    GetPPIScreen (&ppiScr.x, &ppiScr.y);
    wxSize page = g_pageSetupData->GetPaperSize();
    page.x = static_cast<int> (page.x * ppiScr.x / 25.4);
    page.y = static_cast<int> (page.y * ppiScr.y / 25.4);
    m_pageRect = wxRect (0,
                         0,
                         page.x,
                         page.y);

    // get margins informations and convert to printer pixels
    wxPoint pt = g_pageSetupData->GetMarginTopLeft();
    int left = pt.x;
    int top = pt.y;
    pt = g_pageSetupData->GetMarginBottomRight();
    int right = pt.x;
    int bottom = pt.y;

    top = static_cast<int> (top * ppiScr.y / 25.4);
    bottom = static_cast<int> (bottom * ppiScr.y / 25.4);
    left = static_cast<int> (left * ppiScr.x / 25.4);
    right = static_cast<int> (right * ppiScr.x / 25.4);

    m_printRect = wxRect (left,
                          top,
                          page.x - (left + right),
                          page.y - (top + bottom));

    // count pages
    while (HasPage (*maxPage)) {
        m_printed = m_edit->FormatRange (0, m_printed, m_edit->GetLength(),
                                       dc, dc, m_printRect, m_pageRect);
        *maxPage += 1;
    }
    if (*maxPage > 0) *minPage = 1;
    *selPageFrom = *minPage;
    *selPageTo = *maxPage;
}

bool TASEditorTextPrint::HasPage (int WXUNUSED(page)) {

    return (m_printed < m_edit->GetLength());
}

bool TASEditorTextPrint::PrintScaling (wxDC *dc){

    // check for dc, return if none
    if (!dc) return false;

    // get printer and screen sizing values
    wxSize ppiScr;
    GetPPIScreen (&ppiScr.x, &ppiScr.y);
    if (ppiScr.x == 0) { // most possible guess 96 dpi
        ppiScr.x = 96;
        ppiScr.y = 96;
    }
    wxSize ppiPrt;
    GetPPIPrinter (&ppiPrt.x, &ppiPrt.y);
    if (ppiPrt.x == 0) { // scaling factor to 1
        ppiPrt.x = ppiScr.x;
        ppiPrt.y = ppiScr.y;
    }
    wxSize dcSize = dc->GetSize();
    wxSize pageSize;
    GetPageSizePixels (&pageSize.x, &pageSize.y);

    // set user scale
    float scale_x = (float)(ppiPrt.x * dcSize.x) /
                    (float)(ppiScr.x * pageSize.x);
    float scale_y = (float)(ppiPrt.y * dcSize.y) /
                    (float)(ppiScr.y * pageSize.y);
    dc->SetUserScale (scale_x, scale_y);

    return true;
}
*/
#endif // wxUSE_PRINTING_ARCHITECTURE

