/////////////////////////////////////////////////////////////////////////////
// Name:        TASPanel.h
// Purpose:     Declares base panel and 3DPanel classes
// Author:      Matthew J. Patitz
// Copyright:   Copyright 2008, 2009 Matthew J. Patitz
// Licence:
//    This file is part of ISU TAS.
//
//    ISU TAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    ISU TAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with ISU TAS.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////////////

#ifndef TASPanel_H
#define TASPanel_H

#include <wx/dcclient.h>
#include <wx/dynarray.h>
#include <wx/button.h>
#include <wx/checkbox.h>
#include <wx/defs.h>
#include <wx/frame.h>
#include <wx/sizer.h>
#include <wx/radiobox.h>
#include <wx/thread.h>
#include <wx/popupwin.h>
#include <wx/scrolwin.h>
#include "TASSimFrm.h"
#include "TASEditor.h"
#include "TASTiles.h"


// forward declarations
class TASApp;
class TASEditorFrame;
class TASOGLCanvas;

class TASPanelBase : public wxScrolledWindow
{
public:
  TASPanelBase(TASSimulatorFrame *parent,
                 wxWindowID id = -1,
                 const wxPoint& pos = wxDefaultPosition,
                 const wxSize& size = wxDefaultSize,
                 long style = wxTAB_TRAVERSAL | wxNO_BORDER | wxScrolledWindowStyle,
                 const wxString& name = wxPanelNameStr);

  ~TASPanelBase();
  virtual void OnMouse(wxMouseEvent& WXUNUSED(event)) {};

  virtual wxString GetPositionStatusText() { return wxString(_T("")); };

  virtual void SetViewableSpace(const TASCoords& tcMin, const TASCoords& tcMax,
                                int iViewType, bool bForceRecalc = false);
  bool CoordsInNeighborhood(TASCoords& coordsIn);

  void ScrollToCenter(float xPct, float yPct, bool bAdjustExtra);        //  Attempts to scroll the window so that the
                                                                         //  specified percentage values of the virtual
                                                                         //  area are centered (this is called from the
                                                                         //  overview window so bAdjustExtra should be true
  void ScrollToCenter(TASCoords coords, bool bAdjustExtra = false);
  void ScrollToCenter(int xPixel, int yPixel);
  void GetTileLocation(TASCoords& coords, int* x, int* y);

  virtual float GetScaleFact() = 0;
  virtual float GetScaleStep() = 0;
  virtual float SetScaleFact(float WXUNUSED(fScaleFact), TASCoords* WXUNUSED(pCtrCoords) = NULL, bool WXUNUSED(bKeepRelativePosition) = false) { return 0.0f; };

  virtual void UpdateSpace(bool bEntire = false) = 0;

  TASSimulatorFrame*     m_pSimFrame;   //  Pointer to the parent frame

  size_t        m_lastDrawnIter;        //  The last iteration (epoch) that was drawn
  int           m_iViewBy;
  bool          m_bShowWireframe;       //  Toggle gridlines on/off (defaults to off)

  bool          m_bShowMouseOver;
  bool          m_bShowFrontier;

  TASCoords     m_coordsCurrCell;       //  The cell that the mouse is currently over
  TASCoords     m_coordsSelCell;        //  The cell that has been selected by a click or double-click

  TASCoords     m_coordsMinVis;
  TASCoords     m_coordsMaxVis;
  TASCoords     m_coordsNbhdCenter;

  virtual inline int GetTileWidth(int WXUNUSED(col))  { return m_tileWidth; };
  virtual inline int GetTileHeight(int WXUNUSED(row)) { return m_tileHeight; };

  int           m_tileWidth;             //  Width in pixels of a tile dimensions (pre-calculated for efficiency)
  int           m_tileHeight;            //  Height in pixels of a tile dimensions (pre-calculated for efficiency)

  TASTileType   m_ttFrontierTile;       //  The tile type for frontier locations

protected:
  bool          m_bNeedRedraw;          //  To be set when a redraw of the CA space is required

private:
  
  DECLARE_CLASS(TASPanelBase)
  DECLARE_EVENT_TABLE()
};

class TAS3DPanel : public TASPanelBase
{
public:
  TAS3DPanel(TASSimulatorFrame *parent,
             wxWindowID id = -1,
             const wxPoint& pos = wxDefaultPosition,
             const wxSize& size = wxDefaultSize,
             long style = wxTAB_TRAVERSAL | wxNO_BORDER | wxScrolledWindowStyle,
             const wxString& name = wxPanelNameStr);
  ~TAS3DPanel();

  void SetViewableSpace(const TASCoords& tcMin, const TASCoords& tcMax,
                        int iViewType, bool bForceRecalc = false);
  void UpdateSpace(bool bEntire);
  wxString GetPositionStatusText();

  float GetScaleFact();
  float GetScaleStep() { return 25; };

  TASOGLCanvas*  m_pCanvas;
};

#endif // TASPanel_H
